#include <BWAPI.h>
#include <BWTA.h>
#include <time.h>
#include "MacroUnitControl.h"
#include "MicroUnitControl.h"
#include "Vector2.h"


using namespace std;
using namespace BWAPI;
using namespace Helper;
using namespace ICEStarCraft;

StrategyReaction *theMentalManager = NULL;

StrategyReaction* StrategyReaction::create() 
{
	if (theMentalManager) return theMentalManager;
	else return theMentalManager = new StrategyReaction();
}

void StrategyReaction::destroy()
{
	if (theMentalManager) delete theMentalManager;
}


StrategyReaction::StrategyReaction()
{
	this->mChokePoint = ChokepointManager::create();
	this->bom = NULL;
	this->mental = NULL;
	this->onceCommand = new issueOnce();
	this->mDefendTeam.clear();
	this->DefendTeamSize = 0;
	this->btCount = 0;
	this->receivedBTPositions.clear();
	this->allEnemyUnits.clear();
	this->goOutGroup.clear();
	this->CCPosition =BWTA::getStartLocation(Broodwar->self())->getPosition();
	this->highGroundTankLimitation = 0;
	this->nextSetPoint = Positions::None;
	this->lastScanTime = 0;
	lastTourMapTime=0;
	lastAttackPosition.clear();
	this->scm=NULL;
	this->desEnemyMainBase = false;
	this->lastScanPosition = Positions::None;
	this->mInfo = NULL;
	this->searchingUnits.clear(); 
	this->allAttackUnits.clear();
	this->myBattleUnits.clear();
	this->enemyBattleUnits.clear();
	std::set<BaseLocation*> aBL = BWTA::getBaseLocations();
	for each(BaseLocation* bl in aBL){
		this->positionTOattack.insert(make_pair(bl->getPosition(),make_pair(false,0)));
	}
	this->scanTableFlag = false;
	this->groupCenter = NULL;
	this->gf = GameFlow::create();
	unitGroupCenterLimitation = 0;
	vultureGroup.clear();
	startTP = TilePositions::None;
	endTP = TilePositions::None;
	this->attTar.first = "No_AtkTar";
	this->attTar.second=Position(0,0);
	lastMineStart=TilePositions::None;
	lastMineEnd=TilePositions::None;
	TankDefBase.clear();
	TankDrop.clear();
	baseManager=BaseManager::create();
	occupyFlag= false;
	battleC.clear();
	theMarine = NULL;
	toGoIn = NULL;
	//ttBlockTeam.clear();
	//ttrescueTeam.clear();
	this->gatherPoint = Positions::None;
	this->theDropship = NULL;
	this->dropPos = TilePositions::None;
	this->dropBaseCen = TilePositions::None;
	this->tankLoaded = false;
	this->tankSieged = false;
	this->TankDrop.clear();
}

void StrategyReaction::setUsefulManagers(BuildOrderManager* bom)
{
	this->mental= MentalClass::create();
	this->bom = bom;
	this->mChokePoint = ChokepointManager::create();
	this->scm =ScoutManager::create();
	this->eInfo = EnemyInforManager::create();
	this->mInfo = MyInforManager::create();
}

void StrategyReaction::defendDK()
{
	if(this->bom->getPlannedCount(UnitTypes::Terran_Missile_Turret,250)<3){
		this->bom->buildAdditional(1,UnitTypes::Terran_Missile_Turret,300,(TilePosition)mChokePoint->getThirdChoke()->getCenter());
		this->bom->buildAdditional(1,UnitTypes::Terran_Missile_Turret,280,(TilePosition)mChokePoint->getSecondChoke()->getCenter());
		this->bom->buildAdditional(1,UnitTypes::Terran_Missile_Turret,260,(TilePosition)mChokePoint->getFirstChoke()->getCenter());
	}

	UnitGroup all = SelectAll()(isCompleted)(canAttack).not(isWorker,isBuilding);
	for each(Unit* u in all){
		this->onceCommand->attackMoveOnce(u,this->mChokePoint->getSecondChoke()->getCenter(),32*3);
	}
}

void StrategyReaction::defendDropArmy()
{
	if(Broodwar->getFrameCount()%24==0)
		this->allVisibleEnemy = SelectAllEnemy()(isVisible);
	int mpriority = 1000;
	if (receivedBTPositions.size()==0 || Broodwar->getFrameCount()%(24*10)==0){
		receivedBTPositions = mChokePoint->firstCPBuilding();
		for (std::set<TilePosition>::iterator i = receivedBTPositions.begin(); i!= receivedBTPositions.end();i++){
			if(this->bom->getPlannedCount(UnitTypes::Terran_Missile_Turret,800)<6){
				this->bom->buildAdditional(1,UnitTypes::Terran_Missile_Turret,mpriority,*i);
				mpriority-=30;
			}
		}		
	}
	if (Broodwar->getFrameCount()%(24*10)==0){
		UnitGroup tempGroup = SelectAll()(Marine,Vulture,Siege_Tank);
		int i = 0;
		int j = 0;
		int vi = 0;//vulture num
		int ti = 0;//tank num
		if (tempGroup.size()<=12){
			DefendTeamSize = 8;
			vi=3;
			ti=2;
		}
		if (tempGroup.size()>12){
			DefendTeamSize = 12;
			vi=6;
			ti=3;
		}
		if (tempGroup.size()>0){
			for (std::set<Unit*>::const_iterator it=tempGroup.begin();it!= tempGroup.end();it++){
				Unit* _it = (*it);
				if ((int)mDefendTeam.size()>=DefendTeamSize)
					break;
				else if (_it->getType() == UnitTypes::Terran_Vulture){
					if(mDefendTeam.size()==0)
						mDefendTeam.insert(_it);
					else{
						for each(Unit* u in mDefendTeam){
							if(u->getType()== UnitTypes::Terran_Vulture){
								i++;
								continue;
							}						
						}
						if (i<vi){
							if(mDefendTeam.find(_it)==mDefendTeam.end())
								mDefendTeam.insert(_it);
						}
					}						
				}
				else if (_it->getType() == UnitTypes::Terran_Siege_Tank_Tank_Mode){
					//if this is the first unit ,then add into set
					if(mDefendTeam.size()==0)
						mDefendTeam.insert(_it);
					else{
						//count num of tanks in this team 
						for each(Unit* u in mDefendTeam){
							if(u->getType()== UnitTypes::Terran_Siege_Tank_Tank_Mode || u->getType()== UnitTypes::Terran_Siege_Tank_Siege_Mode){
								++j;
								continue;
							}				
						}
						if (mDefendTeam.find(_it)==mDefendTeam.end()&& j<ti){
							mDefendTeam.insert(_it);
						}
					}				
				}

				else if (_it->getType() == UnitTypes::Terran_Marine){
					if (mDefendTeam.find(_it)==mDefendTeam.end())
						mDefendTeam.insert(_it);
				}	
			}
		}

		for each(Unit* u in mDefendTeam){		
			if (u->exists()){
				if (u->getType() == UnitTypes::Terran_Siege_Tank_Tank_Mode){
					if(Broodwar->self()->hasResearched(TechTypes::Tank_Siege_Mode)){
						onceCommand->siegeOnce(u,BWTA::getStartLocation(Broodwar->self())->getPosition());	
					}
					else if(u->getPosition().getDistance(BWTA::getStartLocation(Broodwar->self())->getRegion()->getCenter())>15){
						onceCommand->attackMoveOnce(u,BWTA::getStartLocation(Broodwar->self())->getRegion()->getCenter(),15);	
					}
				}
				else{
					if (u->getLastCommand().getType()==UnitCommandTypes::Attack_Move && Broodwar->getFrameCount()%(24*20)!=0)
						continue;
					if (u->getPosition().getApproxDistance((Position)Broodwar->self()->getStartLocation())>(3500))
						onceCommand->moveOnce(u,BWTA::getStartLocation(Broodwar->self())->getRegion()->getCenter(),15);
					if (u->isAttacking()||u->getOrder()==Orders::AttackMove)
						continue;
					else
						u->attack(BWTA::getStartLocation(Broodwar->self())->getRegion()->getCenter());
					/*if (u->getLastCommand().getType()!=UnitCommandTypes::Attack_Move||
					(u->getLastCommand().getType()==UnitCommandTypes::Attack_Move && !u->isAttacking() && Broodwar->getFrameCount()-mLastCommandTime>24*10)){
					onceCommand->attackMoveOnce(u,BWTA::getStartLocation(Broodwar->self())->getRegion()->getCenter(),15);	
					}*/		
				}		
			}

			else if(mDefendTeam.size()>0 && !u->exists()){
				mDefendTeam.erase(u);
				break;
			}
			for (std::set<Unit*>::iterator i = allVisibleEnemy.begin(); i!= allVisibleEnemy.end();i++){
				Unit* eunit = *i;
				if(eunit->getPosition().getApproxDistance((Position)Broodwar->self()->getStartLocation())<=3200){
					for each(Unit* mu in mDefendTeam){
						if (mu->getOrder()==Orders::AttackMove||mu->getOrder()==Orders::AttackUnit||mu->isAttacking())
							continue;
						else
							mu->attack(eunit->getPosition());
					}
				}			
			}
		}
	}	
}


void StrategyReaction::allUnitAttack(Unit* eu,Position p)
{
	if (Broodwar->getFrameCount()%24==0)
	{
		//if the tile position is visible and enemy does not exist, then return
		if (!eu->exists())
		{
			if (Broodwar->isVisible(eu->getTilePosition()))
				return;
		}

		// set gatherPoint and gather units before the attack
		if (this->mental->goAttack 
			&& (this->attTar.first == "MainBase" || this->attTar.first == "Only_MainBase" || this->attTar.first == "Expansion" || this->attTar.first == "Enemy_Army"))
		{
			BWTA::Region* atkReg = BWTA::getRegion(this->attTar.second);
			if (atkReg)
			{
				Position mapCenter = Position(Broodwar->mapWidth()*32/2,Broodwar->mapHeight()*32/2);
				int px = (int)(atkReg->getCenter().x()*0.5 + mapCenter.x()*0.5);
				int py = (int)(atkReg->getCenter().y()*0.5 + mapCenter.y()*0.5);
				this->gatherPoint = Position(px,py).makeValid();

				Broodwar->drawCircleMap(gatherPoint.x(),gatherPoint.y(),20,Colors::Purple,true);
			}
			else
			{
				this->gatherPoint = Positions::None;
			}	
		}
		else
		{
			this->gatherPoint = Positions::None;
		}

		if (this->mental->waitBeforeAttack && this->gatherPoint != Positions::None)
		{
			if (allAttackUnits.inRadius(32*10,gatherPoint).size() < 0.8 * allAttackUnits.size())
			{ 
				// gather units and send 1 unit to scout
				/*if ((toGoIn || this->mental->scoutBeforeAttack) && !toGoIn->exists())
				{	
				this->mental->scoutBeforeAttack = false;
				toGoIn = NULL;
				}*/

				bool flag = (allAttackUnits.inRadius(32*10,gatherPoint).size() >= 0.2 * allAttackUnits.size()) && !this->mental->scoutBeforeAttack;
				int minD = 9999;
				for each (Unit* u in allAttackUnits)
				{	
					if (toGoIn && u == toGoIn)
					{
						u->attack(p);
						continue;
					}

					if (flag && u->isCompleted() && !u->isLoaded() && u->getType() != UnitTypes::Terran_Siege_Tank_Siege_Mode 
						&& u->getType() != UnitTypes::Terran_Siege_Tank_Tank_Mode&& !u->getType().isBuilding() && u->getPosition().getApproxDistance(p) < minD)
					{
						minD = u->getPosition().getApproxDistance(p);
						toGoIn = u;
					}

					if (u->getType() == UnitTypes::Terran_Siege_Tank_Tank_Mode || u->getType() == UnitTypes::Terran_Siege_Tank_Siege_Mode)
						tankAttackMode(u,gatherPoint);
					else if (u->getType()==UnitTypes::Terran_Science_Vessel)
						ScienceVesselBehavior(u);
					else if (u->getPosition().getApproxDistance(this->gatherPoint) > 32*3)
					{
						if (!u->isAttackFrame())
							u->attack(this->gatherPoint);
					}
					else
						u->holdPosition();
				}
				if (toGoIn && !this->mental->scoutBeforeAttack)
				{
					this->mental->scoutBeforeAttack = true;
					//Broodwar->printf("%s goes to attack",toGoIn->getType().getName().c_str());
					toGoIn->attack(p);
				}
				return;
			}
			else
			{ // let units attack enemy
				if (this->mental->scoutBeforeAttack)
				{
					this->mental->waitBeforeAttack = false;
				}
			}
		}

		//check enemy type, if enemy is flyer
		if (eu && (eu->getType().isFlyer()||eu->isLifted()))
		{
			for each(Unit* u in allAttackUnits)
			{
				if (u->getType().airWeapon()==WeaponTypes::None)
					continue;
				else
				{
					if (u->getLastCommand().getTargetPosition()==u->getTargetPosition() && !u->isIdle())
						continue;
					/*	else if (u->getType()==UnitTypes::Terran_Marine && Broodwar->enemy()->getRace()==Races::Terran && Broodwar->getFrameCount()<=24*60*8)
					continue;*/
					else if(u->getGroundWeaponCooldown()!=0 || u->getAirWeaponCooldown()!=0)
						continue;	
					else if (Unit* best = getBestAttackTartget(u))
					{
						if (u->isAttacking() && u->getTarget() && u->getTarget()==best)
							continue;
						else
							u->attack(best);
					}						
					else if (u->isAttacking()||u->isAttackFrame())
						continue;		
					else
						u->attack(p);								
				}
			}
		}
		else
		{				
			for each(Unit* u in this->allAttackUnits)
			{
				Position safe = isUnderInvAtk(u);
				if (safe != Positions::None)
				{
					if (u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
					{
						u->unsiege();
						continue;
					}						
					else
					{
						u->move(safe);
						//Broodwar->printf("move to safe position (%d, %d)", safe.x(), safe.y());
						//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),safe.x(),safe.y(),Colors::Red);
						continue;
					}					
				}

				if (this->groupCenter && u->getType()!=UnitTypes::Terran_Marine && Broodwar->enemy()->getRace()!=Races::Terran)
				{
					//keep formation
					//if this unit too far from group center
					//&& u->getLastCommand().getType()!=UnitCommandTypes::Attack_Move
					//wounded units run away
					//if (u->isUnderAttack() && (u->getHitPoints()/u->getType().maxHitPoints()<=1/3) && Broodwar->getFrameCount()%24==0)
					if (u->isUnderAttack() && (u->getHitPoints()/u->getType().maxHitPoints()<=1/3))
					{
						u->move(*groupCenter);
						continue;
					}				
				}

				//fight mode
				if (u->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode || u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
					tankAttackMode(u,p);//tankattack;
				else if (u->getType()==UnitTypes::Terran_Science_Vessel)
					ScienceVesselBehavior(u);				
				else
				{
					if (Broodwar->getFrameCount()%24*3==0)
					{
						if (u->getLastCommand().getTargetPosition()==u->getTargetPosition() && !u->isIdle())
							continue;
						else if(u->getGroundWeaponCooldown()!=0 || u->getAirWeaponCooldown()!=0)
							continue;	
						else if (Unit* best = getBestAttackTartget(u))
						{
							if (u->isAttacking() && u->getTarget() && u->getTarget()==best)
								continue;
							else
								u->attack(best);
						}						
						else if (u->isAttacking()||u->isAttackFrame())
							continue;		
						else
							u->attack(p);						
					}		
				}
			}
		}
	}
}


void StrategyReaction::tankAttackMode(Unit* tk,Position p)
{
	//Position safe = isUnderInvAtk(tk);
	//if (safe!=Positions::None){
	//	if (tk->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode){
	//		tk->unsiege();
	//		return;
	//	}						
	//	else{
	//		tk->move(safe);
	//		return;
	//	}					
	//}
	//else 
	if (tk->getGroundWeaponCooldown()!=0)
		return;


	else if (tk->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
	{
		// _T_
		// tanks should not use siege mode near narrow choke point
		BWTA::Chokepoint* nearestCp = BWTA::getNearestChokepoint(tk->getPosition());
		if (nearestCp && nearestCp->getWidth() < 120 && tk->getPosition().getApproxDistance(nearestCp->getCenter()) < 32*4)
		{
			tk->unsiege();
			return;
		}
		double MaxValue=0;
		Unit* bestTarget = NULL;
		Position tp = tk->getPosition();
		int attackRange = UnitTypes::Terran_Siege_Tank_Siege_Mode.groundWeapon().maxRange();
		UnitGroup enemyInRange = SelectAllEnemy()(isVisible).not(isFlyer,isLifted).inRadius(UnitTypes::Terran_Siege_Tank_Siege_Mode.groundWeapon().maxRange()*1.2,tk->getPosition());
		for each(Unit* u in enemyInRange){
			if (u->getPlayer()==Broodwar->self())
				continue;
			else if(u->getTilePosition().getDistance(tk->getTilePosition())<=3 && this->allAttackUnits.inRadius(32*6,tk->getPosition()).size()<4){
				tk->unsiege();
				return;
			}

			else if(u->isVisible() && !u->getType().isFlyer()&&!u->isLifted()){
				double attackValue = 0;
				if (u->getType()==UnitTypes::Protoss_High_Templar || u->getType()==UnitTypes::Zerg_Defiler 
					|| u->getType()==UnitTypes::Zerg_Lurker || u->getType()==UnitTypes::Protoss_Reaver ||u->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode
					||u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode){
						bestTarget = u;
						break;
				}
				else if (u->getType().canAttack()){
					if(u->exists()){
						double hp = u->getHitPoints()+u->getShields();
						double dpf = u->getType().groundWeapon().damageAmount()*1.0/u->getType().groundWeapon().damageCooldown();
						double price = u->getType().mineralPrice()+u->getType().gasPrice();
						//dpf 50% , HP  35%  ,price 15%
						attackValue = dpf*0.5*price*0.15/hp*0.35;
					}												
					else
						attackValue = 0;
					if (MaxValue==0 || attackValue>MaxValue){
						MaxValue = attackValue;
						bestTarget = u;
					}
				}

				else if (!u->getType().canAttack()){
					if(u->exists()){
						double hp = u->getHitPoints()+u->getShields();
						double price = u->getType().mineralPrice()+u->getType().gasPrice();
						//HP  60%  ,price 40%
						attackValue = price*0.4/hp*0.6*0.5;
					}												
					else
						attackValue = 0;

					if (MaxValue==0 || attackValue>MaxValue){
						MaxValue = attackValue;
						bestTarget = u;
					}
				}
			}	
		}
		if (enemyInRange.empty())
			tk->unsiege();
		else{
			if (bestTarget){
				tk->attack(bestTarget);
			}				

			else
				tk->attack(p);
		}
	}

	else if (tk->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode){		
		if (tk->isAttacking() || tk->isAttackFrame())
		{	
			return;
		}
		else{
			int surroundingEnemy=0;
			UnitGroup enemyInRange = SelectAllEnemy()(isVisible).not(isFlyer,isLifted).inRadius(UnitTypes::Terran_Siege_Tank_Tank_Mode.groundWeapon().maxRange()*1.3,tk->getPosition());
			if(enemyInRange.empty()){
				if(Broodwar->getFrameCount()%24==0)
					tk->attack(p);
			}			
			else{
				//check if enemy close to this tank
				for each(Unit* u in enemyInRange){
					if (u->getPlayer()==Broodwar->self())
						continue;
					else if (!u->getType().isFlyer() && !u->isLifted() && u->getTilePosition().getDistance(tk->getTilePosition())<=4 && u->isVisible())
						surroundingEnemy++;
				} 
				//if true
				if (surroundingEnemy>0){
					if(Unit* best = getBestAttackTartget(tk)){
						if(tk->getGroundWeaponCooldown()==0)
							tk->attack(best);
					}					
					else if(tk->getGroundWeaponCooldown()==0)
						tk->attack(p);
				}
				// _T_
				// tanks should not use siege mode near narrow choke point
				else
				{
					BWTA::Chokepoint* nearestCp = BWTA::getNearestChokepoint(tk->getPosition());
					if (nearestCp && (nearestCp->getWidth() > 120 || tk->getPosition().getApproxDistance(nearestCp->getCenter()) > 32*4))
						tk->siege();
					else if (!nearestCp)
						tk->siege();
				}
			}		
		}
	}
}

void StrategyReaction::groupUnitAttack(UnitGroup group, Position p)
{
	for each(Unit* u in group){
		if(u->isAttacking() || u->getOrder()==Orders::AttackMove|| u->getOrder()==Orders::AttackTile|| u->getOrder()==Orders::AttackUnit)
			continue;
		else
			u->attack(p);
	}
}

void StrategyReaction::DestroySpecifiedUnit(UnitGroup group,Unit* target)
{
	for each(Unit* u in group){
		if (u->isAttacking() || u->getOrder()==Orders::AttackMove|| u->getOrder()==Orders::AttackTile|| u->getOrder()==Orders::AttackUnit)
			continue;
		else
			u->attack(target);
	}
}

void StrategyReaction::onUnitDestroy(Unit* u){
	if (u == NULL)
	{
		return;
	}
	
	battleC.erase(u);
	if (Broodwar->self()->isEnemy(u->getPlayer()) && u->getType().isBuilding()){
		if(this->scm->enemyStartLocation &&  u->getTilePosition().getDistance(this->scm->enemyStartLocation->getTilePosition())<=18)
			this->desEnemyMainBase = true;
	}
	//for DefendDropArmy Turret detect
	if (u->getType()==UnitTypes::Terran_Missile_Turret){
		TilePosition see = u->getTilePosition();
		for each(TilePosition tp in receivedBTPositions){
			if((int)u->getTilePosition().getDistance(tp)<10)
				this->btCount--;
		}
	}

	if (searchingUnits.find(u)!=searchingUnits.end())
		searchingUnits.erase(u);
	if (this->allAttackUnits.find(u)!=this->allAttackUnits.end())
		this->allAttackUnits.erase(u);

	if (u->getPlayer() == Broodwar->self() && u->getType() == UnitTypes::Terran_Dropship)
	{
		tankLoaded = false;
		tankSieged = false;
		TankDrop.clear();
	}
	
	//remove this unit from tank drop team
	TankDrop.erase(u);
	//remove this unit from tank def team
	TankDefBase.erase(u);
	//remove from base protecting team
	for each(BaseClass* bc in this->baseManager->getBaseSet()){
		bc->protectingArmy.erase(u);
	}

}


void StrategyReaction::normalTVPDefend()
{	
	BattlecruiserBehavior();
	//if (!allAttackUnits.empty()){
	//	for each(Unit* u in allAttackUnits){
	//		if (Unit* bestTar = getBestAttackTartget(u))
	//			Broodwar->drawTextMap(u->getPosition().x(),u->getPosition().y()-5,"%s",bestTar->getType().c_str());
	//	}
	///*	for each(Unit* tk in this->allAttackUnits){
	//		Broodwar->drawTextMap(tk->getPosition().x(), tk->getPosition().y(), "%d", tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint()));
	//	}*/
	//	//Broodwar->drawCircleMap(mChokePoint->getMyBestSetPoint().x(), mChokePoint->getMyBestSetPoint().y(), 5, Colors::Cyan);
	//}

	/*for each(Unit* e in SelectAllEnemy()(canAttack).not(isWorker))
	Broodwar->drawTextMap(e->getPosition().x()-10,e->getPosition().y()-10,"\x19 %s %d",e->getType().groundWeapon().c_str(), e->getType().groundWeapon().damageCooldown());*/

	//Broodwar->drawCircleMap(Broodwar->self()->getStartLocation().x()*32,Broodwar->self()->getStartLocation().y()*32,32*this->unitGroupCenterLimitation,Colors::Purple);
	//Broodwar->drawCircleMap(Broodwar->self()->getStartLocation().x()*32,Broodwar->self()->getStartLocation().y()*32,32*this->mental->sightRangeLimitation,Colors::Yellow);
	//if (this->groupCenter != NULL){
	//	Broodwar->drawCircleMap((*groupCenter).x(),(*groupCenter).y(),3,Colors::Red,true);
	//	Broodwar->drawCircleMap(this->allAttackUnits.getCenter().x(),this->allAttackUnits.getCenter().x(),3,Colors::Green,true);
	//}
	if (!this->mental->goAttack)
		this->unitGroupCenterLimitation = Broodwar->self()->supplyUsed()/10;
	else
		this->unitGroupCenterLimitation = 40;
	//update vulture mining group
	for(std::set<Unit*>::iterator i = vultureGroup.begin();i!= vultureGroup.end();){
		//if ((*i)->isCompleted()&&(*i)->getSpiderMineCount()==0){
		if ((*i)->isCompleted() && ((*i)->getSpiderMineCount()==0 || !SelectAllEnemy(canAttack).not(isFlyer).inRadius((*i)->getType().sightRange(),(*i)->getPosition()).empty()))
		{
			i = vultureGroup.erase(i);
		}
		else{
			i++;
			continue;
		}
	}

	//for clear the dangerous mines
	if (Broodwar->getFrameCount()%24==0 && this->groupCenter && !this->mental->enemyInSight.empty()){
		//if enemey is still far
		if (this->mental->enemyInSight(canAttack).not(isWorker).getCenter().getApproxDistance(*this->groupCenter)>=32*8){
			UnitGroup dangerMine =SelectAll()(Vulture_Spider_Mine).inRadius(32*3.2,*this->groupCenter);
			for each(Unit* mine in dangerMine){
				if (!this->allAttackUnits(canAttack).empty()){
					for each(Unit* u in this->allAttackUnits.not(isSieged)){
						if (u->isAttacking()||u->isAttackFrame()||u->getGroundWeaponCooldown()!=0)
							continue;
						else{
							u->attack(mine);
							break;
						}
					}
				}
			}
		}
	}

	//adding fighters into set
	for each(Unit* u  in Broodwar->self()->getUnits()){
		if (u->isLoaded()||u->getType().isBuilding()||u->getType().isWorker()||!u->isCompleted()||u->getType()==UnitTypes::Terran_Vulture_Spider_Mine
			|| u->getType()==UnitTypes::Terran_Battlecruiser || u->getType()==UnitTypes::Terran_Dropship)
			continue;
		else if (this->TankDrop.find(u)!= this->TankDrop.end())
			continue;
		else if (this->TankDefBase.find(u)!= this->TankDefBase.end())
			continue;
		//else if (this->vultureGroup.find(u) != this->vultureGroup.end())
		//	continue;
		else if (u->getType()==UnitTypes::Terran_Marine &&
			this->mInfo->CountUnitNum(UnitTypes::Terran_Marine,1)<=this->mInfo->CountUnitNum(UnitTypes::Terran_Bunker,2)*4)
			continue;
		else
			this->allAttackUnits.insert(u);
	}
	//for zergling rush timing attack
	if (this->mental->STflag==MentalClass::ZrushZergling && SelectAll()(Marine)(isCompleted,isLoaded).size()>=3){
		this->allAttackUnits+= SelectAll()(Marine)(isCompleted).not(isLoaded);
	}


	//if we are safe now , then divide a vulture group for mining  && !this->mental->goAttack
	//if (this->mental->enemyInSight(canAttack).size()<=10)
	//if(!this->mental->goAttack)
	this->allAttackUnits-=this->vultureGroup;
	//for vulture mining
	//if(!this->vultureGroup.empty()&&Broodwar->self()->hasResearched(TechTypes::Spider_Mines)&&this->mental->enemyInSight.size()<1
	//	&&!this->mental->goAttack && SelectAll()(isCompleted)(Vulture_Spider_Mine).size()<80)
	//	minetest();

	if (!this->vultureGroup.empty() && Broodwar->self()->hasResearched(TechTypes::Spider_Mines)){
		//if we go attack
		if (this->mental->goAttack){	
			if(Broodwar->getFrameCount()%24*5==0 && SelectAllEnemy()(canAttack).not(isWorker,isBuilding).inRadius(32*12,allAttackUnits.getCenter()).size()<=3)
				vultureGroup.initMining((TilePosition)this->allAttackUnits(Siege_Tank).getCenter(),(TilePosition)this->mChokePoint->esecondChokepoint->getCenter());
			minetest();
		}		

		else{
			if(Broodwar->getFrameCount()%24*5==0)
				MiningRouteInitialization();
			if (this->mental->enemyInSight(canAttack).not(isWorker,isBuilding).empty())
				minetest();
		}		
	}

	//locate group center point
	int x = 0;
	int y = 0;
	int count = 0;

	for each(Unit* u in this->allAttackUnits){
		if (TankDefBase.find(u)!=TankDefBase.end()){
			TankDefBase.erase(u);
			break;
		}
		else{
			if (u->getTilePosition().getDistance(Broodwar->self()->getStartLocation())<this->unitGroupCenterLimitation)
				continue;
			else{
				x+=u->getPosition().x();
				y+=u->getPosition().y();
				count++;
			}
		}		
	}

	if (count==0)
		this->groupCenter=NULL;
	if (count>0)
		this->groupCenter = new Position(x/count,y/count);

	//for marine go into bunker
	if (this->mInfo->CountUnitNum(UnitTypes::Terran_Bunker,1)>0){
		//if we have at least a bunker
		std::set<Unit*> mBunker = SelectAll()(isCompleted)(Bunker);
		std::set<Unit*> mMarine = SelectAll()(isCompleted)(Marine).not(isLoaded);
		Unit* theBunker = NULL;
		//if attack flag is false
		if (this->mental->goAttack==false){
			////check if enemy insight
			//if (this->mental->enemyInSight.size()>0){
			//	for each(Unit* bk in mBunker){
			//		
			//		//if we got enemy and the enemy is out of the bunker's range, unload all units in the bunker
			//		if (bk->getLoadedUnits().size()>0 && 
			//			SelectAllEnemy().inRadius(UnitTypes::Terran_Marine.groundWeapon().maxRange(),bk->getPosition()).size()<1){
			//				//if enemies are in our base
			//				if (this->mental->enemyInSight.getNearest(BWTA::getStartLocation(Broodwar->self())->getPosition())->getTargetPosition().getApproxDistance(BWTA::getStartLocation(Broodwar->self())->getPosition())<32*6){
			//					bk->unloadAll();
			//				}							
			//		}
			//		//if the bunker is empty and got enemy in range, then load marine
			//		else if (bk->getLoadedUnits().size()<4 && 
			//			SelectAllEnemy().inRadius(UnitTypes::Terran_Marine.groundWeapon().maxRange(),bk->getPosition()).size()>0){
			//				if (Broodwar->getFrameCount()%24==0){
			//					SelectAll()(isCompleted)(Marine).rightClick(bk);
			//				}
			//		}
			//		else{
			//			if (Broodwar->getFrameCount()%24==0)
			//				SelectAll()(isCompleted)(Marine).rightClick(bk);
			//		}
			//	}
			//}
			//// if there's no enemy
			//else{
			//	for each(Unit* bk in mBunker){
			//		if(bk->getLoadedUnits().size()>=4 || mMarine.size()==0)
			//			continue;
			//		else
			//			SelectAll()(isCompleted)(Marine).not(isLoaded).rightClick(bk);										
			//	}
			//}

			//check how many bunkers we have so far
			//if we only have one bunker
			if (mBunker.size()==1){
				theBunker = *mBunker.begin();
				if (Broodwar->getFrameCount()%18==0 && theBunker)
					SelectAll()(isCompleted)(Marine).rightClick(theBunker);		
			}	
			//if we have more than 1 bunker, then always go inside the one near the 2nd base
			if (mBunker.size()>1){
				//check the bunkers' positions
				for each(Unit* bk in mBunker){
					if (bk->isCompleted() && BWTA::getRegion(bk->getTilePosition()) != BWTA::getRegion(Broodwar->self()->getStartLocation())){
						if (bk->getLoadedUnits().size()>=4)
							continue;
						else{
							SelectAll()(isCompleted)(Marine).not(isLoaded).rightClick(bk);			
						}						
					}

					else{
						if (bk->isCompleted() && bk->getLoadedUnits().size() > 0)						
							bk->unloadAll();
					}					
				}
			}			
		}

		//if go attack flag is true, then all marine go to attack
		else{
			for each(Unit* bk in mBunker){
				if(bk->getLoadedUnits().size()>0)
					bk->unloadAll();
				else
					continue;				
			}				
		}		
	}
	//build turret
	if (Broodwar->getFrameCount()>=24*60*8+24*5 && Broodwar->enemy()->getRace() == Races::Protoss){
		if(this->bom->getPlannedCount(UnitTypes::Terran_Missile_Turret)<2){
			this->bom->build(1,UnitTypes::Terran_Missile_Turret,80,(TilePosition)mChokePoint->getThirdChoke()->getCenter());
			this->bom->build(2,UnitTypes::Terran_Missile_Turret,85,Broodwar->self()->getStartLocation());
		}
	}

	if (Broodwar->getFrameCount()%8==0)
	{
		//--------------------------- mBestSetPoint
		Position setPoint = Positions::None;
		if (Broodwar->enemy()->getRace()==Races::Terran){
			Position mapCenter = Position(Broodwar->mapWidth()*32/2,Broodwar->mapHeight()*32/2);
			Position candidate2 = Position((this->mChokePoint->getFirstChoke()->getCenter().x()+mapCenter.x())/2,(this->mChokePoint->getFirstChoke()->getCenter().y()+mapCenter.y())/2);
			//lift building patrol point
			Position liftDesPos = Positions::None;
			if(this->scm->enemyStartLocation){
				liftDesPos = this->mChokePoint->efirstChokepoint->getCenter();
				Position eFirstChoke = this->mChokePoint->efirstChokepoint->getCenter();
				Position eSecondChoke = this->mChokePoint->esecondChokepoint->getCenter();
				Position eThirdChoke = this->mChokePoint->ethirdChokepoint->getCenter();
				Position gatherPoint = Positions::None;
				Position candidate = Position((eSecondChoke.x()*2/3+mapCenter.x()/3),(eSecondChoke.y()*2/3+mapCenter.y()/3));
				int deadArmy = Broodwar->self()->deadUnitCount(UnitTypes::Terran_Siege_Tank_Tank_Mode)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_Siege_Tank_Siege_Mode)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_Goliath)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_Marine)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_SCV);
				unsigned int enemyTank = this->eInfo->CountEunitNum(UnitTypes::Terran_Siege_Tank_Siege_Mode);
				//Broodwar->drawCircleMap(this->mChokePoint->mBestSetPoint.x(),this->mChokePoint->mBestSetPoint.y(),20,Colors::Cyan,true);
				if((Broodwar->self()->supplyUsed()/2>=60 || this->eInfo->killedEnemyNum > deadArmy)&&
					enemyTank <=7 && (SelectAll()(isCompleted)(Siege_Tank).size()>=enemyTank+3)){
						gatherPoint = candidate;
						liftDesPos = candidate;				
						//allBuildingSetPoint(gatherPoint);
						this->mChokePoint->mBestSetPoint = candidate;

				}
				else if (this->allAttackUnits(Siege_Tank).size()>=3){		
					gatherPoint = candidate2;
					liftDesPos = candidate2;				
					//allBuildingSetPoint(gatherPoint);
					this->mChokePoint->mBestSetPoint = candidate2;
				}

				else{
					if (this->gf->bunkerPosition && 
						BWTA::getRegion(*gf->bunkerPosition) != BWTA::getRegion(Broodwar->self()->getStartLocation())){
							//allBuildingSetPoint((Position)(*this->gf->bunkerPosition));
							this->mChokePoint->mBestSetPoint =(Position)(*this->gf->bunkerPosition);
					}
					else{
						//Position re =  BWTA::getRegion(Broodwar->self()->getStartLocation())->getCenter();
						//allBuildingSetPoint(re);
						if (Broodwar->getFrameCount()<=24*60*5 && !this->mental->marineRushOver)
							this->mChokePoint->mBestSetPoint = mapCenter;
						else
							this->mChokePoint->mBestSetPoint = this->mChokePoint->getThirdChoke()->getCenter();
					}
				}

				//don't need barrack any more
				//SelectAll()(isCompleted)(Marine).size()>=3|| this->gf->bunkerPosition!=NULL || 
				//this->mInfo->CountUnitNum(UnitTypes::Terran_Siege_Tank_Tank_Mode,2)>0
				if (this->mental->marineRushOver && this->mInfo->CountUnitNum(UnitTypes::Terran_Marine,1)>=4){
					//SelectAll()(isCompleted)(Barracks,Engineering_Bay)(isLifted).move(this->mChokePoint->getMyBestSetPoint());
					UnitGroup tmp = SelectAll()(isCompleted)(Barracks,Engineering_Bay)(isLifted);

					for each(Unit* liftU in tmp){
						//if we do not see any enemy
						UnitGroup en = SelectAllEnemy().inRadius(liftU->getType().sightRange() + 32*6, liftU->getPosition())(canAttack).not(isFlyer, isBuilding, isWorker);
						if (en.empty()){
							//if the barrack is close enough to the destination
							//if (liftU->isMoving()) continue;
							if (liftU->getPosition().getApproxDistance(this->mChokePoint->esecondChokepoint->getCenter())<=32){
								//go back when under attack
								if (liftU->isUnderAttack())
									liftU->move(this->allAttackUnits.getCenter());
								else
									continue;
							}							
							else
								liftU->move(this->mChokePoint->esecondChokepoint->getCenter());
						}
						//if we find enemy, just follow the enemy center
						else{
							if (liftU->isUnderAttack())
								liftU->move(this->allAttackUnits.getCenter());
							else {
								UnitGroup enTank = en(Siege_Tank);

								UnitGroup atkUnitInRange = allAttackUnits.inRadius(liftU->getType().sightRange(), liftU->getPosition());
								//Position aUnitCenter = atkUnitInRange.empty()? mChokePoint->esecondChokepoint->getCenter() : allAttackUnits.getCenter();
								Position aUnitCenter = atkUnitInRange.empty()?allAttackUnits.getCenter() : atkUnitInRange.getCenter();
								Position eTankCenter = enTank.empty()? aUnitCenter : enTank.getCenter();
								liftU->move(Position((int)((1.0*eTankCenter.x() + aUnitCenter.x())/2), (int)((1.0*eTankCenter.y() + aUnitCenter.y())/2)));
							}
						}
					}
					SelectAll()(isCompleted)(Barracks,Engineering_Bay).not(isLifted).lift();
				}
			} 
			//if we do not know where is the enemy
			else{			
				if (Broodwar->getFrameCount()<=24*60*5){
					this->mChokePoint->mBestSetPoint = mapCenter;
				}
				else
					this->mChokePoint->mBestSetPoint = this->mChokePoint->getThirdChoke()->getCenter();
			}
		}
		/**************for other races************/
		else{
			if(Broodwar->self()->supplyUsed()/2<=100){
				//if we have bunker
				if (!this->gf->bunkerPosition){
					Position re =  BWTA::getRegion(Broodwar->self()->getStartLocation())->getCenter();
					Position fp =Position((re.x()+this->mChokePoint->getFirstChoke()->getCenter().x())/2,(re.y()+this->mChokePoint->getFirstChoke()->getCenter().y())/2);
					//if the bunker is in main base 
					if(this->gf->bunkerPosition && BWTA::getRegion(*gf->bunkerPosition) == BWTA::getRegion(Broodwar->self()->getStartLocation())){
						//and we don't have any expansion yet,then gather around the bunker
						if (this->mInfo->CountUnitNum(UnitTypes::Terran_Command_Center,2) == 1){
							allBuildingSetPoint(fp);
							this->mChokePoint->mBestSetPoint = fp;
						}
						// if we have expansion, then gather outside to protect expansion
						else{
							allBuildingSetPoint(this->mChokePoint->getSecondChoke()->getCenter());
							this->mChokePoint->mBestSetPoint = this->mChokePoint->getSecondChoke()->getCenter();
						}			
					}
					// if the bunker is not in the main base, then ignore where it is, just gather at the 2nd expansion
					else{
						std::set<Unit*> mBase = SelectAll()(Command_Center);

						for each(Unit* bc in mBase){
							//if the base is at start location, then skip
							if (bc->getTilePosition().getDistance(Broodwar->self()->getStartLocation()) < 3)
								continue;
							else{
								//if the region is 2nd base
								if (BWTA::isConnected((TilePosition)BWTA::getRegion(Broodwar->self()->getStartLocation())->getCenter(),(TilePosition)bc->getRegion()->getCenter())){
									this->mChokePoint->mBestSetPoint = bc->getRegion()->getCenter();
								}			
							}
						}
					}


					if (!SelectAll()(Missile_Turret).inRadius(32*5,this->mChokePoint->getSecondChoke()->getCenter()).empty()){
						allBuildingSetPoint(this->mChokePoint->getSecondChoke()->getCenter());
						this->mChokePoint->mBestSetPoint = this->mChokePoint->getSecondChoke()->getCenter();
					}
					else{
						allBuildingSetPoint(fp);
						this->mChokePoint->mBestSetPoint = fp;
					}		
				}
				else{
					allBuildingSetPoint((Position)(*this->gf->bunkerPosition));
					this->mChokePoint->mBestSetPoint =(Position)(*this->gf->bunkerPosition);
				}
			}			

			if(Broodwar->self()->supplyUsed()/2>100){
				// if safe and we have advantages
				if (this->mInfo->myFightingValue().first > this->eInfo->enemyFightingValue().first*1.2){
					Position eFirstChoke = this->mChokePoint->efirstChokepoint->getCenter();
					Position eSecondChoke = this->mChokePoint->esecondChokepoint->getCenter();
					Position eThirdChoke = this->mChokePoint->ethirdChokepoint->getCenter();
					Position mapCenter = Position(Broodwar->mapWidth()*32/2,Broodwar->mapHeight()*32/2);
					Position gatherPoint = Positions::None;
					Position candidate = Position((eThirdChoke.x()*2/3+mapCenter.x()/3),(eThirdChoke.y()*2/3+mapCenter.y()/3));
					Position candidate2 = Position((this->mChokePoint->getFirstChoke()->getCenter().x()+mapCenter.x())/2,(this->mChokePoint->getFirstChoke()->getCenter().y()+mapCenter.y())/2);
					//if enemy 3rd choke is far enough from eSL
					if (this->scm->enemyStartLocation->getPosition().getApproxDistance(eThirdChoke)>32*22)
					{
						//first check whether we have army on this spot
						if (this->allAttackUnits.inRadius(32*10,candidate).size()>=4)
						{
							//then check enemy size
							if (SelectAllEnemy()(isDetected,canAttack).not(isBuilding,isWorker).inRadius(32*15,candidate).size()<=6)
							{
								gatherPoint =candidate;	
							}
							else
								gatherPoint = candidate2;
						}
						else
							gatherPoint = candidate2;
						//set gather position
						allBuildingSetPoint(gatherPoint);
						this->mChokePoint->mBestSetPoint = gatherPoint;
					}

					//else if eThirdChokePoint is near to Ebase
					else
					{
						gatherPoint = candidate2;
						allBuildingSetPoint(gatherPoint);
						this->mChokePoint->mBestSetPoint = gatherPoint;

					}

				}
				//if we don't have so much army, then gathering at somewhere near base
				else
				{
					if (this->gf->bunkerPosition && BWTA::getRegion(*gf->bunkerPosition) != BWTA::getRegion(Broodwar->self()->getStartLocation())){
						allBuildingSetPoint((Position)(*this->gf->bunkerPosition));
						//this->mChokePoint->mBestSetPoint = BWTA::getNearestBaseLocation(this->mChokePoint->getSecondChoke()->getCenter())->getPosition();
						this->mChokePoint->mBestSetPoint =(Position)(*this->gf->bunkerPosition);
					}
					else
					{
						allBuildingSetPoint((this->mChokePoint->getThirdChoke()->getCenter()));
						//this->mChokePoint->mBestSetPoint = BWTA::getNearestBaseLocation(this->mChokePoint->getSecondChoke()->getCenter())->getPosition();
						this->mChokePoint->mBestSetPoint =(this->mChokePoint->getThirdChoke()->getCenter());
					}
				}		
			}
		}

		// set best set point for other units (not tanks)
		if (this->mChokePoint->mBestSetPoint)
		{
			if (!this->scm->enemyStartLocation)
			{
				this->mChokePoint->mBestSetPoint2 = this->mChokePoint->mBestSetPoint;
			}
			else
			{
				//Position eBase = this->scm->enemyStartLocation->getPosition();
				Position mapCen = Position(Broodwar->mapWidth()*16,Broodwar->mapHeight()*16);
				BWTA::Region* reg   = BWTA::getRegion(this->mChokePoint->mBestSetPoint);
				BWTA::Region* start = BWTA::getRegion(Broodwar->self()->getStartLocation());
				if (!reg || !start)
				{
					this->mChokePoint->mBestSetPoint2 = this->mChokePoint->mBestSetPoint;
				}
				else if (reg != start)
				{
					int minD = 9999;
					Position mBestSetPoint2 = Positions::None;
					for each (BWTA::Chokepoint* cp in reg->getChokepoints())
					{
						if (cp->getCenter().getApproxDistance(mapCen) < minD)
						{
							minD = cp->getCenter().getApproxDistance(mapCen);
							mBestSetPoint2 = cp->getCenter();
						}
					}
					if (mBestSetPoint2 == Positions::None)
					{
						this->mChokePoint->mBestSetPoint2 = this->mChokePoint->mBestSetPoint;
					}
					else
					{
						if (mBestSetPoint2.getApproxDistance(this->mChokePoint->mBestSetPoint) > 32*6)
						{
							Vector2 v = mBestSetPoint2 - this->mChokePoint->mBestSetPoint;
							v	= v * (32.0*6/v.approxLen());
							mBestSetPoint2 = 	v + this->mChokePoint->mBestSetPoint;
							mBestSetPoint2 = mBestSetPoint2.makeValid();
						}
						this->mChokePoint->mBestSetPoint2 = mBestSetPoint2;
					}
				}
				else
				{
					this->mChokePoint->mBestSetPoint2 = this->mChokePoint->mBestSetPoint;
				}

				Broodwar->drawCircleMap(this->mChokePoint->mBestSetPoint.x(),this->mChokePoint->mBestSetPoint.y(),12,Colors::Red,true);
				Broodwar->drawCircleMap(this->mChokePoint->mBestSetPoint2.x(),this->mChokePoint->mBestSetPoint2.y(),8,Colors::Orange,true);
			}
		}
		else
		{
			//Broodwar->printf("No best set point");
		}

		//--------------------------- ATTACK ENEMY

		//select all enemies that we should attack
		//Broodwar->printf("%d",this->mental->enemyInSight.size());
		//if we discover any enemy
		if (this->mental->enemyInSight.size()>0 && Broodwar->enemy()->getRace()!=Races::Terran)
		{
			this->attTar.first = "Enemy_Army";
			this->attTar.second = this->mental->enemyInSight(canAttack).getCenter();
			//if our army size is large enough, we should ignore small enemy group
			if (Broodwar->self()->supplyUsed()/2>=180)
			{
				//if enemy group is large, we should kill them first
				if (this->mental->enemyInSight.size()>=12)
				{
					for each(Unit* en in this->mental->enemyInSight)
					{
						//if visible enemy still far, then attack move
						if (!en||!en->exists())
							continue;
						else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
							continue;
						else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists())
						{
							this->mental->enemyInSight.erase(en);
							break;
						}
						else if (!en->isVisible()||!en->isDetected())
							continue;
						else
							this->allUnitAttack(en,en->getPosition());
					}
				}
			}
			//else we should kill any enemy insight first
			else
			{
				//for enemy center
				//army center
				Position eCenter = this->mental->enemyInSight(canAttack).not(isBuilding,isFlyer,isLifted).getCenter();	
				//non-attack enemy
				Position backupECenter = this->mental->enemyInSight(isDetected).not(canAttack).getCenter();
				if (eCenter==Positions::None)
					eCenter = backupECenter;
				//for my center
				Position mCenter = Positions::None;
				if(this->groupCenter)
					mCenter = *this->groupCenter;
				else
					mCenter = this->allAttackUnits(canAttack).getCenter();
				//for dis
				int centerDis = mCenter.getApproxDistance(eCenter);
				//if our military power is stronger
				if ((this->mInfo->myFightingValue().first >= this->eInfo->enemyFightingValue().first) && Broodwar->self()->supplyUsed()/2>=80)
				{
					//if we are still far from eCenter, or enemy doesn't have can-attack unit
					//then move attack to that position
					if (centerDis >= 32*18 || eCenter == backupECenter)
					{
						for each(Unit* en in this->mental->enemyInSight)
						{
							//if visible enemy still far, then attack move
							if (!en||!en->exists())
								continue;
							else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
								continue;
							else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists())
							{
								this->mental->enemyInSight.erase(en);
								break;
							}
							else if (!en->isVisible()||!en->isDetected())
								continue;
							else
								this->allUnitAttack(en,en->getPosition());
						}
					}
					else
					{
						//else we should be very careful for each movement
						//for unsieged tank
						UnitGroup unsiegedTank = this->allAttackUnits(Siege_Tank).not(isSieged);
						for each(Unit* utk in unsiegedTank)
						{
							//avoid lurker
							Position safe = isUnderInvAtk(utk);
							if (safe!=Positions::None)
							{
								utk->move(safe);
								//Broodwar->printf("move to safe Pos (%d, %d)", safe.x(), safe.y());
								//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),safe.x(),safe.y(),Colors::Red);
								continue;
							}
							else
							{
								if (utk->getPosition().getApproxDistance(mCenter)<=32*6)
									utk->siege();								
								//still far
								else
									tankAttackMode(utk,eCenter);
							}
						}	
						//for siege tank
						for each(Unit* sgtk in this->allAttackUnits(Siege_Tank)(isSieged))
						{
							if (sgtk->getPosition().getApproxDistance(mCenter)<=32*6){
								if (Unit* bt = getBestAttackTartget(sgtk))
									sgtk->attack(bt);							
							}
							else
								tankAttackMode(sgtk,eCenter);
						}
						//for other units
						for each(Unit* u in this->allAttackUnits.not(Siege_Tank))
						{
							keepFormationAttack(u);
						}
					}
				}
				else
				{
					Position spot = Position((this->allAttackUnits.getCenter().x()+this->mental->enemyInSight.getCenter().x())/2,(this->allAttackUnits.getCenter().y()+this->mental->enemyInSight.getCenter().y())/2);
					//if we are near best gathering point
					if (this->allAttackUnits.getCenter().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*10)
					{
						//if enemy still far
						if (this->mental->enemyInSight.getCenter().getApproxDistance(BWTA::getStartLocation(Broodwar->self())->getPosition())>=32*35)
						{
							//for unsieged tank
							for each(Unit* tk in this->allAttackUnits(Siege_Tank).not(isSieged)){
								if(tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*9)
									tk->siege();
								else
									tankAttackMode(tk,this->mChokePoint->getMyBestSetPoint());
							}
							//other units
							for each(Unit* u in this->allAttackUnits.not(Siege_Tank))
							{
								if (u->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*4)
								{
									if(Unit* bt = getBestAttackTartget(u))
										u->attack(bt);
									else
										u->attack(this->mChokePoint->getMyBestSetPoint());
								}
								else
								{
									if(Unit* bt = getBestAttackTartget(u))
										u->attack(bt);
									else
										u->attack(this->mChokePoint->getMyBestSetPoint());
								}					
							}

							//for sieged tank
							for each(Unit* tk in this->allAttackUnits(Siege_Tank)(isSieged)){
								if (tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*9)
									continue;
								else
									tk->unsiege();		
							}
							//this->mChokePoint->getMyBestSetPoint()
						}
						else
						{
							for each(Unit* en in this->mental->enemyInSight)
							{
								//if visible enemy still far, then attack move
								if (!en||!en->exists())
									continue;
								else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
									continue;
								else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists()){
									this->mental->enemyInSight.erase(en);
									break;
								}
								else if (!en->isVisible()||!en->isDetected())
									continue;
								else
									this->allUnitAttack(en,en->getPosition());
							}
						}
					}
					else
					{	
						//if we are near base,then don't need to move
						if (this->allAttackUnits.getCenter().getApproxDistance(BWTA::getStartLocation(Broodwar->self())->getPosition())<=32*50)
						{
							if (this->mental->enemyInSight(canAttack).getCenter().getApproxDistance(BWTA::getStartLocation(Broodwar->self())->getPosition())>=32*40)
							{
								for each(Unit* tk in this->allAttackUnits(Siege_Tank).not(isSieged)){
									if(tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*9)
										tk->siege();
									else
										tankAttackMode(tk,this->mChokePoint->getMyBestSetPoint());
								}
								//other units
								for each(Unit* u in this->allAttackUnits.not(Siege_Tank))
								{
									if (u->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*4){
										if(Unit* bt = getBestAttackTartget(u))
											u->attack(bt);
										else
											u->attack(this->mChokePoint->getMyBestSetPoint());
									}
									else{
										if(Unit* bt = getBestAttackTartget(u))
											u->attack(bt);
										else
											u->attack(this->mChokePoint->getMyBestSetPoint());
									}					
								}
								//for sieged tank
								for each(Unit* tk in this->allAttackUnits(Siege_Tank)(isSieged)){
									if (tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())<=32*9)
										continue;
									else
										tk->unsiege();		
								}
							}
							else
							{
								for each(Unit* en in this->mental->enemyInSight){
									//if visible enemy still far, then attack move
									if (!en||!en->exists())
										continue;
									else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
										continue;
									else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists()){
										this->mental->enemyInSight.erase(en);
										break;
									}
									else if (!en->isVisible()||!en->isDetected())
										continue;
									else
										this->allUnitAttack(en,en->getPosition());
								}
							}						
						}
						//if our army is too far from home, and weaker than enemy 
						else
						{						
							//if we can find any closest enemy expansion, then attack it
							if (!this->eInfo->eBuildingPositionMap.empty()){
								int dis = 0;
								Position attackP=Positions::None;
								Unit* attackTar = NULL;
								for(std::map<Unit*,std::pair<UnitType,Position>>::const_iterator i = this->eInfo->eBuildingPositionMap.begin();i != this->eInfo->eBuildingPositionMap.end();i++){
									if (i->second.first.isResourceDepot()){
										if (dis==0 || i->first->getPosition().getApproxDistance(this->allAttackUnits.getCenter())<dis){
											dis = i->first->getPosition().getApproxDistance(this->allAttackUnits.getCenter());
											attackP = i->second.second;
											attackTar = i->first;
										}
									}
									else
										continue;
								}
								//go and attack
								if(attackTar&&attackP!=Positions::None)
									allUnitAttack(attackTar,attackP);

								else{
									//if there's no closest expansion , then go attack eMainBase
									if(this->scm->enemyStartLocation){
										//for tank
										for each(Unit* tk in this->allAttackUnits(Siege_Tank)){
											tankAttackMode(tk,this->scm->enemyStartLocation->getPosition());
										}
										//for rest
										for each(Unit* u in this->allAttackUnits.not(Siege_Tank)){
											if(Unit* bt = getBestAttackTartget(u))
												u->attack(bt);
											else
												u->attack(this->scm->enemyStartLocation->getPosition());
										}
									}
									//if we even don't know where is the eSL, then attack base to base
									else{
										for each(Unit* en in this->mental->enemyInSight){
											//Broodwar->printf("%s",en->getType().c_str());
											if (!en||!en->exists())
												continue;
											else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
												continue;
											else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists()){
												this->mental->enemyInSight.erase(en);
												break;
											}
											else if (!en->isVisible()||!en->isDetected())
												continue;
											else
												this->allUnitAttack(en,en->getPosition());
										}
									}													
								}							
							}
							//if we can not find any enemy base 
							else{
								for each(Unit* en in this->mental->enemyInSight){
									//Broodwar->printf("%s",en->getType().c_str());
									if (!en||!en->exists())
										continue;
									else if (en->getPosition()==Positions::None || en->getPosition()==Positions::Invalid ||en->getPosition()==Positions::Unknown)
										continue;
									else if (Broodwar->isVisible(en->getTilePosition()) && !en->exists()){
										this->mental->enemyInSight.erase(en);
										break;
									}
									else if (!en->isVisible()||!en->isDetected())
										continue;
									else
										this->allUnitAttack(en,en->getPosition());
								}
							}
						}
					}
				}
			}			
		}

		if (Broodwar->enemy()->getRace()==Races::Terran)
		{
			//for sight range
			UnitGroup tmp = SelectAll()(isCompleted)(Barracks,Engineering_Bay)(isLifted);
			UnitGroup en ;
			//(canAttack)
			if (this->groupCenter)
				en = SelectAllEnemy().inRadius(32*25,*this->groupCenter).not(isLifted,isBuilding, isWorker);
			for each(Unit* liftU in tmp)
			{
				en += SelectAllEnemy().inRadius(liftU->getType().sightRange() + 32*6, liftU->getPosition())(canAttack).not(isFlyer, isBuilding, isWorker);
			}
			en+=this->mental->enemyInSight;
			//for drawing
			this->attTar.first = "Enemy_Army";
			this->attTar.second = en(canAttack).getCenter();
			//if we see enemy
			if (!en.empty()&&this->groupCenter)
			{
				//if enemy still far , go for the enemy
				if (groupCenter->getApproxDistance(en.getCenter())>32*18)
				{
					for each(Unit* tk in allAttackUnits(Siege_Tank))
					{
						tankAttackMode(tk,en.getCenter());
					}

					for each(Unit* ou in allAttackUnits.not(Siege_Tank,Marine))
					{
						if (ou->isMoving()||ou->isAttacking())
							continue;
						else
							ou->attack(en.getCenter());
					}
				}
				//if close enough
				else
				{			
					//compare the fight value
					//&&en(Siege_Tank)(isSieged).size()<8 
					if (this->mInfo->myFightingValue().first >= this->eInfo->enemyFightingValue().first*5 &&
						this->eInfo->CountEunitNum(UnitTypes::Terran_Siege_Tank_Siege_Mode) < 8 &&
						this->mInfo->myDeadArmy<12){
							for each(Unit* tk in allAttackUnits(Siege_Tank))
							{
								tankAttackMode(tk,en.getCenter());
							}

							for each(Unit* ou in allAttackUnits.not(Siege_Tank,Marine))
							{
								if (ou->isMoving()||ou->isAttacking())
									continue;
								else
									ou->attack(en.getCenter());
							}
					}
					else
					{
						int deadArmy = Broodwar->self()->deadUnitCount(UnitTypes::Terran_Siege_Tank_Tank_Mode)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_Siege_Tank_Siege_Mode)+Broodwar->self()->deadUnitCount(UnitTypes::Terran_Goliath);
						if (this->eInfo->CountEunitNum(UnitTypes::Terran_Siege_Tank_Siege_Mode)<4 && deadArmy<=8){
							for each(Unit* ene in en)
							{
								UnitGroup surroundTank = SelectAllEnemy()(isSieged).inRadius(32*6,ene->getPosition());
								if (ene->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
									continue;
								else if (surroundTank.size()>=1)
									continue;
								else
									allUnitAttack(ene,ene->getPosition());								
							}
						}
						else
						{
							if (groupCenter){
								//for unsiege tank
								for each(Unit* tk in allAttackUnits(Siege_Tank).not(isSieged)){
									if (tk->getPosition().getApproxDistance(*groupCenter)>32*3)
										tankAttackMode(tk,*groupCenter);
									else
										tk->siege();
								}
								//for siege tank
								for each(Unit* tk in allAttackUnits(Siege_Tank)(isSieged)){
									if (tk->getPosition().getApproxDistance(*groupCenter)>32*3)
										tk->unsiege();
									else
										continue;
								}
								//for other unit
								for each(Unit* ou in allAttackUnits.not(Siege_Tank,Marine)){
									if (ou->getPosition().getApproxDistance(*groupCenter)>32*4)
										ou->attack(*groupCenter);
									else{
										if (ou->isHoldingPosition())
											continue;
										else
											ou->holdPosition();
									}								
								}
							}
						}												
					}
				}
			}
		}

		//for unit to retreat or defend
		if (this->mental->goAttack == false && this->mental->enemyInSight.size()<1)
		{
			this->attTar.first = "No_AtkTar";
			this->attTar.second = Position(0,0);
			//if no enemy, and the opponent is terran
			if (Broodwar->enemy()->getRace()==Races::Terran)
			{
				//for tank mode
				for each(Unit* tk in this->allAttackUnits(Siege_Tank).not(isSieged))
				{
					//if still far from best Point,attack move to there
					if (tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())>32*3)
						tankAttackMode(tk,this->mChokePoint->getMyBestSetPoint());					
					else
						tk->siege();
				}
				//for siege mode tank
				for each(Unit* tk in this->allAttackUnits(Siege_Tank)(isSieged)){
					//if still far from best Point,unsiege and then move to there
					if (tk->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())>32*3)
						tk->unsiege();					
					else
						continue;
				}

				//for other non-tank units
				for each(Unit* others in this->allAttackUnits.not(Siege_Tank,Marine)){
					//if far from gather point, move to there
					//if (others->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())>32*3)
					//	others->attack(this->mChokePoint->getMyBestSetPoint());
					if (others->getPosition().getApproxDistance(this->mChokePoint->mBestSetPoint2)>32*3)
						others->attack(this->mChokePoint->mBestSetPoint2);
					else
					{
						//if unit is close enough to the gather point, then hold position
						if (others->isHoldingPosition())
							continue;
						else
							others->holdPosition();
					}						
				}
				//for marine
				for each(Unit* mr in SelectAll()(Marine)(isCompleted).not(isLoaded)){
					if (mr->getPosition().getApproxDistance(this->mChokePoint->mBestSetPoint2)>32*2)
						mr->attack(this->mChokePoint->mBestSetPoint2);
					else
						continue;				
				}
			}
			else
			{
				//UnitGroup allFrontArmy = SelectAll()(isCompleted,canAttack).not(isBuilding,isWorker);
				//allFrontArmy-=this->vultureGroup;
				for each(Unit* u in this->allAttackUnits)
				{			
					//if unit too close to enemy location
					//if (this->scm->enemyStartLocation && u->getTilePosition().getDistance(this->scm->enemyStartLocation->getTilePosition())<75){
					//	//if unit is tank, and sieged
					//	if (u->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode||u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
					//		tankAttackMode(u,this->mChokePoint->getMyBestSetPoint());

					//	else{
					//		if (Unit* bt=getBestAttackTartget(u))
					//			u->attack(bt);
					//		else
					//			u->attack(this->mChokePoint->getMyBestSetPoint());
					//	}							
					//}	
					//if unit too far from gathering position
					if (u->getPosition().getApproxDistance(this->mChokePoint->getMyBestSetPoint())>=32*6)
					{
						if (u->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode||u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode)
							tankAttackMode(u,this->mChokePoint->getMyBestSetPoint());
						else
						{
							if (Unit* bt=getBestAttackTartget(u))
								u->attack(bt);
							else
								u->attack(this->mChokePoint->mBestSetPoint2);
						}	
					}
					else
					{
						if (u->getType()==UnitTypes::Terran_Siege_Tank_Tank_Mode && Broodwar->self()->hasResearched(TechTypes::Tank_Siege_Mode))
						{
							//_T_ 9/7
							// tanks should not use siege mode near narrow choke point
							BWTA::Chokepoint* nearestCp = BWTA::getNearestChokepoint(u->getPosition());
							if (nearestCp && nearestCp->getWidth() > 120 || u->getPosition().getApproxDistance(nearestCp->getCenter()) > 32*4)
								u->siege();
							else
								tankAttackMode(u,this->mChokePoint->getMyBestSetPoint());
						}
						else if(this->mInfo->CountUnitNum(UnitTypes::Terran_Bunker,1)>0 &&
							u->getType()==UnitTypes::Terran_Marine && SelectAll(isCompleted)(Marine).size()<=4)
							continue;
						else
							u->holdPosition(this->mChokePoint->mBestSetPoint2);						
					}
				}
			}
		}

		//***********if goattack flag is true******************//
		//***********if goattack flag is true******************//

		if (this->mental->goAttack && this->mental->enemyInSight(canAttack).size()<10)
		{
			//for 8bb marine rush
			if (Broodwar->enemy()->getRace()==Races::Terran && this->allAttackUnits.not(Marine).empty())
			{
				if (this->scm->enemyStartLocation){
					if (theMarine)
						Broodwar->drawTextMap(theMarine->getPosition().x(),theMarine->getPosition().y(),"The_Marine");
					this->attTar.first = "Main_Base";
					this->attTar.second = this->scm->enemyStartLocation->getPosition();
					UnitGroup rushBK = SelectAll()(isCompleted)(Bunker);
					//for rush bunker behavior
					for each(Unit* bk in rushBK){
						UnitGroup mrInBK= SelectAll()(isLoaded)(Marine);
						UnitGroup nearbyAlly= SelectAll().not(isLoaded)(isCompleted)(Marine).inRadius(32*12,bk->getPosition());
						if (theMarine)
							nearbyAlly.erase(theMarine);
						UnitGroup bkTar = SelectAllEnemy().not(isBuilding).inRadius(UnitTypes::Terran_Marine.groundWeapon().maxRange(),bk->getPosition());
						//if no target in attack range,and no ally nearby, send one marine out to attack enemy
						if (bkTar.empty()&& mrInBK.size()>3 && nearbyAlly.empty() && theMarine){
							for each(Unit* mr in mrInBK){
								if (mr->getHitPoints()<15)
									continue;
								//select a healthy marine
								else{							
									theMarine = mr;
									//unload this marine
									bk->unload(theMarine);
									break;
								}
							}
							//for TheMarine behavior
							if (theMarine){							
								UnitGroup wantKillYou = SelectAllEnemy()(canAttack).not(isBuilding).inRadius(32*10,theMarine->getPosition());
								UnitGroup nonBuildingUnit = SelectAllEnemy()(SCV).not(isBuilding).inRadius(32*20,theMarine->getPosition());
								bool dangers = false;
								for each(Unit* wk in wantKillYou){
									if (wk->getTarget()==theMarine){
										dangers = true;
										break;
									}
								}
								//if someone wants to kill this marine, then back to bunker
								if (dangers && !theMarine->isMoving())
									theMarine->rightClick(bk);
								else{
									//else tease enemy
									for each(Unit* tarSCV in nonBuildingUnit){
										//kill constructing scv first
										if (tarSCV->isConstructing()){
											if(theMarine->isAttacking() || theMarine->isMoving())
												continue;
											else{
												theMarine->attack(tarSCV);
												break;
											}

										}
										else{
											if(theMarine->isAttacking() || theMarine->isMoving())
												continue;
											else{
												theMarine->attack(tarSCV);
												break;
											}												
										}
									}
								}
							}							
						}
						else{
							if (mrInBK.size() == 4)
								break;
							if (BWTA::getRegion(this->scm->enemyStartLocation->getTilePosition())==BWTA::getRegion(bk->getTilePosition())) {
								if (bk->getLoadedUnits().size()<4 && Broodwar->getFrameCount()%24==0)
									this->allAttackUnits(Marine).rightClick(bk);
								break;
							}
						}				
					}
					//delete the Marine from unit group
					if (theMarine)
						this->allAttackUnits.erase(theMarine);
					for each(Unit* mr in this->allAttackUnits(Marine)){
						UnitGroup nonBuildingUnit = SelectAllEnemy()(canAttack).not(isBuilding).inRadius(32*20,mr->getPosition());

						if (mr->getTarget() && mr->getTarget()->getType()==UnitTypes::Terran_Bunker)
							continue;
						else if (nonBuildingUnit.empty()){
							if (mr->isStuck()){
								mr->stop();
								continue;
							}
							else if (!mr->isMoving())
								mr->move(this->scm->enemyStartLocation->getPosition());
						}

						else{
							if (mr->getHitPoints()<=20 && mr->isUnderAttack())
								mr->move(this->allAttackUnits(Marine).getCenter());
							else if (mr->getGroundWeaponCooldown()==0){
								int dis = 0;
								Unit* tar = NULL;
								for each(Unit* enu in nonBuildingUnit){
									if (enu->isAttacking()){
										tar = enu;
										break;
									}
									else if (enu->isConstructing()){
										tar = enu;
										break;
									}
									else if (dis==0 || enu->getPosition().getApproxDistance(mr->getPosition())<dis){
										dis =  enu->getPosition().getApproxDistance(mr->getPosition());
										tar = enu;
									}
								}
								if (tar)
									mr->attack(tar);								
							}								
						}
					}
				}				
			}
			else
			{
				//if we already have eSL
				if (this->scm->enemyStartLocation)
				{
					if (this->eInfo->enemyBaseMap.size()>0)
					{
						//if our army is large enough
						if (Broodwar->self()->supplyUsed()/2>=190 ||
							  (Broodwar->self()->supplyUsed()/2>=150 && (this->mInfo->myFightingValue().first>=1.5*this->eInfo->enemyFightingValue().first)))
						{
							//then attack enemy main base directly
							int dis = 0;
							Position attackP=Positions::None;
							Unit* attackTar = NULL;
							Unit* backupTar = NULL;
							Position backupP=Positions::None;

							for(std::map<Unit*,EnemyInforManager::eBaseData>::iterator i = this->eInfo->enemyBaseMap.begin();i != this->eInfo->enemyBaseMap.end();i++)
							{
								if (i->second.isMainBase)
								{
									dis = i->second.position.getApproxDistance(this->allAttackUnits.getCenter());
									attackP=i->second.position;
									attackTar = i->first;
								}							
								else
								{
									//find enemy nearest expansion as backup target
									if (dis==0 || i->second.position.getApproxDistance(this->scm->enemyStartLocation->getPosition())<dis)
									{
										dis = i->second.position.getApproxDistance(this->scm->enemyStartLocation->getPosition());
										backupTar = i->first;
										backupP = i->second.position;
									}
								}
							}

							//if enemy still has main base
							if (attackTar && attackP!=Positions::None)
							{
								allUnitAttack(attackTar,attackP);
								this->attTar.first = "MainBase";
								this->attTar.second = attackP;
							}
							else
							{
								//we already destroyed enemy's main base or can't find enemy main base
								//go and destroy other expansions
								if (backupTar && backupP!=Positions::None && backupTar!=attackTar)
								{
									allUnitAttack(backupTar,backupP);
									this->attTar.first = "Expansion";
									this->attTar.second = backupP;
								}
								else
								{
									//else attack the buildings we know on the map
									for(std::map<Unit*,std::pair<UnitType,Position>>::const_iterator i = this->eInfo->eBuildingPositionMap.begin();i != this->eInfo->eBuildingPositionMap.end();i++)
									{
										if (!i->first)
											continue;
										else if (i->second.second == Positions::Invalid || i->second.second == Positions::None || i->second.second == Positions::Unknown)
											continue;
										/*	else if (!i->first->exists() && Broodwar->isVisible(i->first->getTilePosition()))
										continue;*/
										else if (Broodwar->isVisible(i->first->getTilePosition()) && SelectAllEnemy()(isBuilding).inRadius(32*8,i->second.second).size()<1)
											continue;									
										else
										{
											allUnitAttack(i->first,i->second.second);
											this->attTar.first = i->second.first.c_str();
											this->attTar.second = i->second.second;
											break;
										}						
									}
								}
							}
						}	
						//our army is not large enough, so start from enemy's expansion
						else
						{
							//attack from enemy expansion
							int dis = 0;
							Position attackP = Positions::None;
							Position backupP=Positions::None;
							Unit* attackTar = NULL;
							Unit* backupTar = NULL;
							for(std::map<Unit*,EnemyInforManager::eBaseData>::iterator i = this->eInfo->enemyBaseMap.begin();i != this->eInfo->enemyBaseMap.end();i++)
							{
								if (i->second.isMainBase)
								{
									backupTar = i->first;
									backupP = i->second.position;
								}
								else if (dis==0 || i->second.position.getApproxDistance(this->scm->enemyStartLocation->getPosition())>dis)
								{
									dis = i->second.position.getApproxDistance(this->scm->enemyStartLocation->getPosition());
									attackP=i->second.position;
									attackTar = i->first;
								}
							}
							//if we can find the bestAttackBase 
							if (attackTar && attackP!=Positions::None)
							{
								this->attTar.first = "Expansion";
								this->attTar.second = attackP;
								allUnitAttack(attackTar,attackP);
							}
							//else attack the enemy main base instead
							else
							{
								if (backupTar && backupP!=Positions::None)
								{
									allUnitAttack(backupTar,backupP);
									this->attTar.first = "Only_MainBase";
									// should this be "backupP" ?
									// this->attTar.second = attackP;
									this->attTar.second = backupP;
								}
								//if we even can't find main base, then attack rest buildings
								else
								{
									for(std::map<Unit*,std::pair<UnitType,Position>>::const_iterator i = this->eInfo->eBuildingPositionMap.begin();i != this->eInfo->eBuildingPositionMap.end();i++)
									{
										if (!i->first)
											continue;
										else if (i->second.second == Positions::Invalid || i->second.second == Positions::None || i->second.second == Positions::Unknown)
											continue;
										/*	else if (!i->first->exists() && Broodwar->isVisible(i->first->getTilePosition())){
										continue;
										}*/
										else if (Broodwar->isVisible(i->first->getTilePosition()) && SelectAllEnemy()(isBuilding).inRadius(32*8,i->second.second).size()<1)
											continue;	
										else{
											allUnitAttack(i->first,i->second.second);
											this->attTar.first = i->second.first.c_str();
											this->attTar.second = i->second.second;
											break;
										}						
									}
								}

							}		
						}
					}
					//if we can not find any base, then attack the  buildings on the map
					else
					{
						for(std::map<Unit*,std::pair<UnitType,Position>>::const_iterator i = this->eInfo->eBuildingPositionMap.begin();i != this->eInfo->eBuildingPositionMap.end();i++){
							if (!i->first)
								continue;
							else if (i->second.second == Positions::Invalid || i->second.second == Positions::None || i->second.second == Positions::Unknown)
								continue;
							/*	else if (!i->first->exists() && Broodwar->isVisible(i->first->getTilePosition())){
							continue;
							}*/
							else if (Broodwar->isVisible(i->first->getTilePosition()) && SelectAllEnemy()(isBuilding).inRadius(32*8,i->second.second).size()<1)
								continue;	
							else
							{
								allUnitAttack(i->first,i->second.second);
								this->attTar.first = i->second.first.c_str();
								this->attTar.second = i->second.second;
								break;
							}						
						}
					}
				}
				//if we still don't know where is the eSL, then attack the eBuildings on the map
				else
				{
					for(std::map<Unit*,std::pair<UnitType,Position>>::const_iterator i = this->eInfo->eBuildingPositionMap.begin();i != this->eInfo->eBuildingPositionMap.end();i++)
					{
						if (!i->first)
							continue;
						else if (i->second.second == Positions::Invalid || i->second.second == Positions::None || i->second.second == Positions::Unknown)
							continue;
						/*else if (!i->first->exists() && Broodwar->isVisible(i->first->getTilePosition())){
						continue;
						}*/
						else if (Broodwar->isVisible(i->first->getTilePosition()) && SelectAllEnemy()(isBuilding).inRadius(32*8,i->second.second).size()<1)
							continue;	
						else{
							allUnitAttack(i->first,i->second.second);
							this->attTar.first = i->second.first.c_str();
							this->attTar.second = i->second.second;
							break;
						}						
					}
				}

				if (this->desEnemyMainBase)
				{
					if (!Broodwar->enemy()->isDefeated())
						findRestEnemy();
				}
			}

		}
	}

	// for drop
	if (Broodwar->getFrameCount()%24 == 0)
	{
		for each (Unit* tank in TankDrop)
		{
			if (this->dropPos != TilePositions::None && !tank->isLoaded() && !tank->isSieged() && tank->getTilePosition().getDistance(this->dropPos) < 3)
			{
				tank->siege();
				if (!tankSieged)
				{
					tankSieged = true;
				}
			}
		}
		// move dropship to enlarge tanks' vision
		if (tankSieged && theDropship && theDropship->exists() && this->dropBaseCen != TilePositions::None)
		{
			if (theDropship->isUnderAttack())
			{
				theDropship->move(Position(this->dropPos));
			}
			else
			{
				theDropship->move(Position(this->dropBaseCen));
			}
		}
		
		UnitGroup dropships = SelectAll()(isCompleted)(Dropship);
		if (!dropships.empty() && !tankSieged)
		{
			if (!theDropship || (theDropship && !theDropship->exists()))
			{
				theDropship = *(dropships.begin());
			}

			if (this->dropPos != TilePositions::None)
			{
				//if (theDropship->getLoadedUnits().size() < 2 && theDropship->getTilePosition().getDistance(this->dropPos) > 1)
				if (TankDrop.size() < 2)
				{
					// select 2 tanks to load
					UnitGroup tanks = SelectAll()(Siege_Tank)(isCompleted).not(isLoaded)(HitPoints,">=",120);
					if (tanks.size() > 2)
					{
						tankLoaded = false;
						int first  = 9999;
						int second = 9999;
						Unit* firstTank  = NULL;
						Unit* secondTank = NULL;
						for each (Unit* tank in tanks)
						{
							if (theDropship->getPosition().getApproxDistance(tank->getPosition()) < first)
							{
								second = first;
								first = theDropship->getPosition().getApproxDistance(tank->getPosition());
								secondTank = firstTank;
								firstTank  = tank;
							}
							else if (theDropship->getPosition().getApproxDistance(tank->getPosition()) < second)
							{
								second = theDropship->getPosition().getApproxDistance(tank->getPosition());
								secondTank = tank;
							}
						}
						if (firstTank && firstTank->exists() && !firstTank->isLoaded())
						{
							allAttackUnits.erase(firstTank);
							TankDrop.insert(firstTank);
						}
						if (secondTank && secondTank->exists() && !secondTank->isLoaded())
						{
							allAttackUnits.erase(secondTank);
							TankDrop.insert(secondTank);
						}
					}
				}
				else if (!tankLoaded && theDropship->getLoadedUnits().size() < 2)
				{
					// load tanks into dropship
					for each (Unit* tank in TankDrop)
					{
						if (tank->isSieged())
						{
							tank->unsiege();
						}
						else
							theDropship->load(tank,true);
					}
				}
				else
				{
					tankLoaded = true;
					// go to drop position
					if (theDropship->getTilePosition().getDistance(this->dropPos) > 2)
					{
						if (!(theDropship->getLastCommand().getType() == UnitCommandTypes::Move && theDropship->getLastCommand().getTargetPosition() == Position(this->dropPos)))
						{
							theDropship->move(Position(this->dropPos));
						}
					}
					else
					{
						if (theDropship->getLoadedUnits().size() > 0 &&theDropship->getLastCommand().getType() != UnitCommandTypes::Unload_All)
						{
							//tankUnloaded = true;
							Broodwar->printf("unload");
							theDropship->unloadAll();
						}
					}
				}
			}
			else
			{
				// find the position to drop
				if (this->eInfo->enemyBaseMap.size() > 1)
				{
					for(std::map<Unit*,EnemyInforManager::eBaseData>::iterator i = this->eInfo->enemyBaseMap.begin();i != this->eInfo->enemyBaseMap.end();i++)
					{
						int n = 0;
						int sumX = 0, sumY = 0;

						for each (TilePosition tp in this->mChokePoint->dropReg)
						{
							if (tp.getDistance(i->second.tPosition) <= 16)
							{
								n++;
								sumX += tp.x();
								sumY += tp.y();
							}
						}
						if (n > 0)
						{
							this->dropPos = TilePosition(sumX/n,sumY/n);
							this->dropBaseCen = i->second.tPosition;
						}
					}
				}
			}
		}
	}

	//-------------------------------- BATTLE CONTROLLER
	// add units to myBattleUnits group
	myBattleUnits.clear();
	for each (Unit* u in allAttackUnits)
	{
		if (u->isCompleted() && !u->isLoaded() && !u->getType().isBuilding() && (u->isAttacking() || u->getLastCommand().getType() == UnitCommandTypes::Attack_Unit || u->getOrder() == Orders::AttackUnit || u->isUnderAttack()))
		{
			myBattleUnits.insert(u);
		}
	}
	if (myBattleUnits.empty())
	{
		return;
	}

	Position mCen = myBattleUnits.getCenter();
	UnitGroup rest = allAttackUnits - myBattleUnits;
	rest += this->vultureGroup;
	for each (Unit* u in rest)
	{
		if (u->isCompleted() && !u->isLoaded() && !u->getType().isBuilding() && u->getPosition().getApproxDistance(mCen) <= 32*10)
		{
			myBattleUnits.insert(u);
			if (vultureGroup.find(u) != vultureGroup.end())
			{
				vultureGroup.erase(u);
			}
		}
	}
	if(myBattleUnits.empty())
	{
		return;
	}

	// add units to enemyBattleUnits group
	enemyBattleUnits.clear();
	for each (Unit* e in SelectAllEnemy())
	{
		UnitType et = e->getType();
		if (!e->isCompleted() || (et.isBuilding() && !et.canAttack()) || et.isWorker() || et.isFlyer() || et == UnitTypes::Terran_Vulture_Spider_Mine || et == UnitTypes::Zerg_Overlord
			|| et == UnitTypes::Zerg_Larva || et == UnitTypes::Protoss_Interceptor || e->isLoaded() || !e->isVisible() || !e->isDetected())
		{
			continue;
		}
		else if (e->getPosition().getApproxDistance(mCen) <= 32*20)
		{
			enemyBattleUnits.insert(e);
		}
	}

	// control our units
	if (this->attTar.first == "MainBase" || this->attTar.first == "Only_MainBase" || this->attTar.first == "Expansion" || this->attTar.first == "Enemy_Army")
	{
		BWTA::Region* targetReg = BWTA::getRegion(this->attTar.second);
		if (targetReg)
		{
			BWTA::Region* mReg = BWTA::getStartLocation(Broodwar->self())->getRegion();
			if (this->attTar.first == "Enemy_Army" &&
				(Broodwar->getFrameCount() <= 24*60*7 || ICEStarCraft::Helper::isConnected(targetReg,mReg) || targetReg == mReg))
			{
				NormalBattleController();
			}
			else
			{
				BWTA::Chokepoint* theChokePoint = NULL;
				int minDis = 99999;
				for each (BWTA::Chokepoint* cp in targetReg->getChokepoints())
				{
					if (cp->getCenter().getApproxDistance(myBattleUnits.getCenter()) < minDis)
					{
						minDis = cp->getCenter().getApproxDistance(myBattleUnits.getCenter());
						theChokePoint = cp;
					}
				}
				int x = theChokePoint->getCenter().x();
				int y = theChokePoint->getCenter().y();
				//Broodwar->drawTriangleMap(x,y-20,x-17,y+10,x+17,y+10,Colors::Brown);
				//Broodwar->drawTriangleMap(x-17,y-10,x+17,y-10,x,y+20,Colors::Brown);

				if (theChokePoint->getWidth() < 120)
				{
					ChokepointBattleController(this->attTar.second,theChokePoint);
				}
				else
				{
					NormalBattleController();
				}
			}
		}
	}
	else if (this->attTar.first != "No_AtkTar")
	{
		NormalBattleController();
	}

	// avoid dangerous psionic storm
	for each (Unit* u in this->allAttackUnits)
	{
		if (!u->isCompleted() || u->isLoaded() || u->getType().isBuilding())
			continue;

		if (u->getType() == UnitTypes::Terran_Siege_Tank_Siege_Mode)
			continue;

		Position pos = Positions::None;
		Vector2 velocity = Vector2(0,0);
		for each (Unit* e in this->enemyBattleUnits)
		{
			if (e->getType() == UnitTypes::Protoss_High_Templar && e->getOrder() == Orders::CastPsionicStorm)
			{
				Position tarPos = e->getOrderTargetPosition();
				if (tarPos != Positions::None && u->getPosition().getApproxDistance(tarPos) <= 32*4)
				{
					velocity += PFFunctions::getVelocitySource(tarPos,u->getPosition()) * 1000;
				}
			}
		}

		for each (Bullet* b in Broodwar->getBullets())
		{
			if (b->getType() == BulletTypes::Psionic_Storm || b->getType() == BulletTypes::Psionic_Shockwave_Hit)
			{
				if (u->getPosition().getApproxDistance(b->getPosition()) <= 32*5)
				{
					velocity += PFFunctions::getVelocitySource(b->getPosition(),u->getPosition()) * 1000;
				}
			}
		}

		if (velocity != Vector2(0,0))
		{
			velocity = velocity * (256.0 / velocity.approxLen());
			pos = velocity + u->getPosition();
			pos = pos.makeValid();
			u->move(pos);
		}
	}		
}

void StrategyReaction::ChokepointBattleController(Position atkTar,BWTA::Chokepoint* theChokePoint)
{
	for each (Unit* u in myBattleUnits)
	{
		if (u->getType().isFlyer())
		{
			continue;
		}

		Position des = MicroUnitControl::getDestination2(u,myBattleUnits,atkTar,theChokePoint,this->mChokePoint);

		if (des && des.isValid())
		{
			if (!(u->isMoving() && !u->isIdle() && u->getLastCommand().getType() == UnitCommandTypes::Move && u->getLastCommand().getTargetPosition() == des))
			{
				u->move(des);
				//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),des.x(),des.y(),Colors::Green);
				//Broodwar->drawCircleMap(des.x(),des.y(),2,Colors::Green);
			}
		}
	}
}

void StrategyReaction::NormalBattleController()
{
	if (enemyBattleUnits.empty())
	{
		return;
	}

	BWTA::Region* eReg = BWTA::getRegion(enemyBattleUnits.getCenter());
	BWTA::Region* mReg = NULL;

	for each (Unit* u in myBattleUnits)
	{
		if (u->getType().isFlyer() || u->getType() == UnitTypes::Terran_Siege_Tank_Siege_Mode || u->getType() == UnitTypes::Terran_Siege_Tank_Tank_Mode)
		{
			// flying unit or tank
			continue;
		}
		if (u->getType().groundWeapon() != WeaponTypes::None && u->getType().groundWeapon().maxRange() <= 32)
		{
			// melee unit
			continue;
		}

		mReg = BWTA::getRegion(u->getPosition());
		BWTA::Chokepoint* cp = ICEStarCraft::Helper::getChokePoint(mReg,eReg);
		if (!cp || (cp && cp->getWidth() < 200
			&& Broodwar->getGroundHeight(u->getPosition().x() >> 5, u->getPosition().y() >> 5) != Broodwar->getGroundHeight(enemyBattleUnits.getCenter().x() >> 5, enemyBattleUnits.getCenter().y() >> 5)))
		{
			continue;
		}

		int range = u->getType().sightRange();
		UnitGroup enemyInRange = enemyBattleUnits.inRadius((double)range,u->getPosition());
		if (!u->isAttacking() && enemyInRange.empty())
		{
			Broodwar->drawTextScreen(u->getPosition().x()+5,u->getPosition().y(),"move");

			Position targetPosition = MicroUnitControl::getDestination(u,myBattleUnits,enemyBattleUnits);
			if (targetPosition && targetPosition.isValid())
			{
				if (!(u->isMoving() && u->getLastCommand().getType() == UnitCommandTypes::Move && u->getLastCommand().getTargetPosition() == targetPosition))
				{
					u->move(targetPosition);
					//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),targetPosition.x(),targetPosition.y(),Colors::Green);
					//Broodwar->drawCircleMap(targetPosition.x(),targetPosition.y(),2,Colors::Green);
				}
			}
			else
			{
				u->attack(myBattleUnits.getCenter());
			}	
		}
		else if (!enemyInRange.empty())
		{
			Unit* target = getBestAttackTartget(u);
			if(target)
			{	
				if(!u->isAttackFrame() && !(u->getLastCommand().getType() == UnitCommandTypes::Attack_Unit && u->getLastCommand().getTarget() == target))
				{
					u->attack(target);
					//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),target->getPosition().x(),target->getPosition().y(),Colors::Yellow);
				}
			}
			else
			{
				u->attack(enemyInRange.getCenter());
			}	
		}
	}
}

void StrategyReaction::ChokePointBlockSolution()
{
	goOutGroup.clear();
	for each(Unit* u in Broodwar->self()->getUnits()){
		if (u->getType().isBuilding()&&u->isCompleted())
			continue;
		else{
			Position currentPosition = u->getPosition();
			Position TargetPosition = u->getTargetPosition();

			//double TargetGroundDis = BWTA::getGroundDistance(BWTA::getStartLocation(Broodwar->self())->getTilePosition(),(TilePosition)TargetPosition);
			double TargetDisFromBase = TargetPosition.getDistance(BWTA::getStartLocation(Broodwar->self())->getPosition());
			double stdDis1= TargetPosition.getDistance(mChokePoint->getFirstChoke()->getCenter());
			double stdDis2= TargetPosition.getDistance(mChokePoint->getSecondChoke()->getCenter());
			double stdDis3= TargetPosition.getDistance(mChokePoint->getThirdChoke()->getCenter());

			//for first chokepoint
			if((TargetDisFromBase>stdDis1 && TargetDisFromBase >23*32 &&TargetDisFromBase <26*32)||
				(currentPosition.getDistance(BWTA::getStartLocation(Broodwar->self())->getPosition())>23*32 && TargetDisFromBase<20*32)&&u->isMoving() && !u->isAttacking())
				goOutGroup.insert(u);
			//for 2nd chokepoint
			else if (stdDis1>stdDis2 && TargetDisFromBase >26*32 && u->isMoving()&& !u->isAttacking()){
				goOutGroup.insert(u);
			}
			else if (stdDis2>stdDis3 && TargetDisFromBase >26*32 && u->isMoving()&& !u->isAttacking()){
				goOutGroup.insert(u);
			}
		}
	}

	UnitGroup firstChokeGroup = SelectAll()(isCompleted).not(isBuilding,isWorker,isAttacking).inRadius(3*32,mChokePoint->getFirstChoke()->getCenter())-goOutGroup;
	UnitGroup secondChokeGroup = SelectAll()(isCompleted).not(isBuilding,isWorker,isAttacking).inRadius(4*32,mChokePoint->getSecondChoke()->getCenter())-goOutGroup;
	UnitGroup thirdChokeGroup = SelectAll()(isCompleted).not(isBuilding,isWorker,isAttacking).inRadius(4*32,mChokePoint->getThirdChoke()->getCenter())-goOutGroup;


	for each(Unit* u in this->goOutGroup){
		if(firstChokeGroup.size()>0&&firstChokeGroup.find(u)==firstChokeGroup.end()){
			for each(Unit* fu in firstChokeGroup){
				if (fu->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode){
					fu->unsiege();
				}
			}
			if(u->getPosition().getApproxDistance(firstChokeGroup.getCenter())<7*32 && firstChokeGroup.size()>=1){
				//go to base
				if (u->getPosition().getApproxDistance(this->CCPosition)>u->getPosition().getApproxDistance(mChokePoint->getFirstChoke()->getCenter())
					&& u->getPosition().getApproxDistance(this->CCPosition)>20*32){
						firstChokeGroup.rightClick(CCPosition);
				}
				//go out from base
				else
					firstChokeGroup.rightClick(mChokePoint->getSecondChoke()->getCenter());
			}
		}
		if(secondChokeGroup.size()>0&&secondChokeGroup.find(u)==secondChokeGroup.end()){

			if (u->getPosition().getApproxDistance(secondChokeGroup.getCenter())<7*32 && secondChokeGroup.size()>13){
				secondChokeGroup.rightClick(secondChokeGroup.getCenter());
				mLastCommandTime = Broodwar->getFrameCount();
			}
		}
		if (thirdChokeGroup.size()>0&&thirdChokeGroup.find(u)==thirdChokeGroup.end()){
			if (u->getPosition().getApproxDistance(thirdChokeGroup.getCenter())<7*32 && thirdChokeGroup.size()>13){
				thirdChokeGroup.rightClick(thirdChokeGroup.getCenter());
			}
		}		
	}
}




void StrategyReaction::onUnitDiscover(Unit* unit)
{
	if (unit->getPlayer()==Broodwar->self() && unit->getType()== UnitTypes::Terran_Vulture && Broodwar->enemy()->getRace()==Races::Protoss){
		this->vultureGroup.insert(unit);
	}
	//for tvt tank defend base team
	if (unit->getPlayer()==Broodwar->self() && unit->getType()== UnitTypes::Terran_Siege_Tank_Tank_Mode && Broodwar->enemy()->getRace()==Races::Terran){
		UnitGroup tankGroup = SelectAll()(isCompleted)(Siege_Tank);
		unsigned int teamSize = baseManager->getBaseSet().size();
		if (tankGroup.size()>=10 && TankDefBase.size()<teamSize){
			TankDefBase[unit]=Positions::None;
		}
	}
	if (unit->getType()==UnitTypes::Terran_Battlecruiser && unit->getPlayer()==Broodwar->self())
		battleC.insert(unit);
}



UnitGroup StrategyReaction::highGroundSiegeTank(UnitGroup myTank,int limitNum)
{
	TilePosition tp = mChokePoint->getTankSiegePosition();
	Position* p = new Position(tp.x()*32,tp.y()*32);
	int countSiege = (int)SelectAll()(isCompleted)(isSieged)(Siege_Tank).inRadius(32*3,*p).size();
	for each(Unit* tank in myTank){
		if (Broodwar->self()->hasResearched(TechTypes::Tank_Siege_Mode)){
			if (countSiege==limitNum)
				break;					
			else if(countSiege>limitNum){
				tank->unsiege();
				countSiege--;
				continue;
			}			
			else if(tank->getLastCommand().getType()==UnitCommandTypes::Siege)
				continue;
			else if (tank->getPosition().getApproxDistance(*p)>32*3){
				tank->move(*p);
				continue;
			}
			else if(tank->getPosition().getApproxDistance(*p)<=32*3 && !tank->isSieged()){
				tank->siege();
				continue;
			}	
		}
		else if(!Broodwar->self()->hasResearched(TechTypes::Tank_Siege_Mode)){
			onceCommand->attackMoveOnce(tank,mChokePoint->getSecondChoke()->getCenter(),60);
		}

	}

	//delete high ground siege tank
	if((int)SelectAll()(isCompleted)(isSieged)(Siege_Tank).inRadius(32*3,*p).size()>=limitNum){
		UnitGroup returnGroup = SelectAll()(isCompleted).not(SCV,isBuilding) - SelectAll()(isCompleted)(isSieged)(Siege_Tank).inRadius(32*3,*p);
		return returnGroup;
	}

	else
		return SelectAll()(isCompleted).not(SCV,isBuilding,Siege_Tank);	
}

void StrategyReaction::unitAvoidMine(UnitGroup ug,Position p)
{	
	for each(Unit* u in ug){
		bool danger =false;
		std::set<Unit*> temp = u->getUnitsInRadius(32*3);
		for each(Unit* su in temp){			
			if (su->getType()==UnitTypes::Terran_Vulture_Spider_Mine && SelectAllEnemy()(isCompleted).not(isBuilding,isFlyer).inRadius(32*3,u->getPosition()).size()>0)
				u->move(this->mChokePoint->getFirstChoke()->getCenter());	
		}
	}		
}

void StrategyReaction::allBuildingSetPoint(Position setposition){
	if (Broodwar->getFrameCount()%(24*5)==0){
		UnitGroup allArmyFactory = SelectAll()(Barracks, Factory,Starport);
		allArmyFactory.setRallyPoint(setposition);
	}
}

void StrategyReaction::ScanInvisibleEnemy()
{
	std::set<Unit*> allUnits = Broodwar->getAllUnits();
	std::set<Unit*> myScanner = SelectAll()(Comsat_Station)(Energy,">=",50);
	if (myScanner.size()==0){
		for each(Unit* eu in allUnits){
			if(eu->getPlayer()!=Broodwar->enemy())
				continue;
			else if(!eu->isDetected() && eu->getType().canAttack()){
				//Broodwar->printf("invisible enemy! Build scanner as soon as possible!");
				if (this->bom->getPlannedCount(UnitTypes::Terran_Comsat_Station,120)<2)
					this->bom->build(2,UnitTypes::Terran_Comsat_Station,120);
			}
		}
	}
	else if (myScanner.size()>0 && Broodwar->getFrameCount()%24*5==0){
		std::set<Unit*> myArmy = SelectAll()(isCompleted).not(isWorker,isBuilding)+SelectAll()(isCompleted)(Bunker)(LoadedUnitsCount,">=",1);
		std::set<Unit*>::iterator scanner = myScanner.begin();
		for each(Unit* eu in allUnits){
			if(eu->getPlayer()!=Broodwar->enemy())
				continue;
			//eu->getTarget()!=NULL && eu->getTarget()->getPlayer()==Broodwar->self()
			else if(!eu->isDetected() && eu->getType().canAttack()){		
				// avoid wasting scanner energy to scan near position in short time
				//(eu->getPosition().getApproxDistance(this->lastScanPosition) <= 32*6 && Broodwar->getFrameCount() - this->lastScanTime <= 24*10)
				if(eu->getPosition().getApproxDistance(this->lastScanPosition) <= 32*6 && Broodwar->getFrameCount() - this->lastScanTime <= 24*10)
					return;
				else{
					for each(Unit* army in myArmy){
						if (army->getPosition().getApproxDistance(eu->getPosition()) <= army->getType().groundWeapon().maxRange()*1.1){
							(*scanner)->useTech(TechTypes::Scanner_Sweep,eu->getPosition());
							this->lastScanTime = Broodwar->getFrameCount();
							this->lastScanPosition = eu->getPosition();
							break;
						}
						else
							continue;
					}			
				}
			}
		}
	}	
}


void StrategyReaction::findRestEnemy()
{
	if (Broodwar->getFrameCount()%24==0){
		/**********for scv to find rest enemy*******/
		this->scm->setScoutNum(2);
		this->scm->SCVScout(ScoutManager::EnemyExpansion);
		/**********for scanner to find*******/
		//check if all the positions have already been scanned
		int scanCounter = 0;
		for (std::map<Position,std::pair<bool,int>>::iterator i = this->positionTOattack.begin(); i != this->positionTOattack.end(); i++){
			if (i->second.first)
				scanCounter++;
		}
		//if so ,then reset	
		if(scanCounter ==(int)this->positionTOattack.size() && Broodwar->getFrameCount()%24*10==0){
			for (std::map<Position,std::pair<bool,int>>::iterator i = this->positionTOattack.begin(); i != this->positionTOattack.end(); i++){
				i->second.first = false;
			}
		}
		//Processing scan task
		for (std::map<Position,std::pair<bool,int>>::iterator i = this->positionTOattack.begin(); i != this->positionTOattack.end(); i++){
			std::set<Unit*> myScanner;
			myScanner = SelectAll()(Comsat_Station)(Energy,">=",50);
			//if the position is currently visible, then don't need to scan
			if (Broodwar->isVisible((TilePosition)i->first)){
				//make sure we never scan this position
				if(i->second.first){
					i->second.second = Broodwar->getFrameCount();
					continue;
				}
				else{
					i->second.first = true;
					i->second.second = Broodwar->getFrameCount();
					continue;
				}
			}
			//if this position has just been scanned
			else if (i->second.first)
				continue;
			//position need to scan
			else{
				for each(Unit* sc in myScanner){
					if ((int)myScanner.size()<=2 && sc->getEnergy()<100)
						return;
					else if (sc->getEnergy()<50)
						continue;
					else{
						int energy=sc->getEnergy();
						//scan this baselocation
						if(sc->useTech(TechTypes::Scanner_Sweep,i->first)){
							i->second.first = true;
							i->second.second = Broodwar->getFrameCount();
							return;
						}
					}
				}

			}
		}
	}
}


void StrategyReaction::ScienceVesselBehavior(Unit* u)
{
	UnitGroup unDetectEnemy = SelectAllEnemy()(canAttack).not(isDetected).inRadius(32*60,this->allAttackUnits.getCenter());
	//check if is underattack
	if (u->isUnderAttack()){
		//if we have anti-air units in our group
		if (this->allAttackUnits(Marine,Goliath).size()>0)
			u->move(this->allAttackUnits(Marine,Goliath).getCenter());
		else
			u->move(this->mChokePoint->getMyBestSetPoint());
	}
	else{
		//for undetected enemy	
		if (unDetectEnemy.empty()){
			if (!allAttackUnits(isAttacking).empty()){
				for each(Unit* head in allAttackUnits(isAttacking)){
					if (head->getTarget()){
						if (!u->isMoving()){
							u->move(head->getPosition());	
							break;
						}						
					}
				}
			}
			else{

				if (!u->isMoving())
					u->move(this->allAttackUnits.getCenter());
			}						
		}			
		else{
			for each(Unit* ude in unDetectEnemy){
				if (!ude || !ude->exists())
					continue;
				else{
					if (u->getPosition().getApproxDistance(ude->getPosition())>32*3){
						if (u->getTargetPosition()!=ude->getPosition())
							u->move(unDetectEnemy.getCenter());
					}
					else{
						if (!u->isHoldingPosition())
							u->holdPosition();
					}
				}
			}

		}	
	}	

	//for use tech
	if (Broodwar->getFrameCount()%24*3==0 && u->getEnergy()>=100 && unDetectEnemy.empty()){
		UnitGroup nearAllies =	this->allAttackUnits.inRadius(u->getType().sightRange()*2,u->getPosition());
		for each(Unit* ally in nearAllies){
			if(ally->isUnderAttack() && ally->getHitPoints() >= ally->getType().maxHitPoints()/3 && !ally->isDefenseMatrixed()){
				u->useTech(TechTypes::Defensive_Matrix,ally);
				break;
			}
		}
	}
}

void StrategyReaction::showScanPositionTable()
{
	if(this->scanTableFlag){	
		Broodwar->drawTextScreen(400,20,"\x0E| Position - Scanned - LastScannedTime |");
		int uline = 1;
		for(std::map<Position,std::pair<bool,int>>::const_iterator i = this->positionTOattack.begin();i != this->positionTOattack.end();i++){
			Broodwar->drawTextScreen(400,20+16*uline,"\x0F|   ( %d ,%d )  -  %d  -  %d   |",i->first.x(),i->first.y(),i->second.first,i->second.second);
			uline++;
		}
	}
}

bool StrategyReaction::setScanTableFlag(bool b)
{
	this->scanTableFlag = b;
	return this->scanTableFlag;
}
void StrategyReaction::minetest()
{
	MiningRouteInitialization();
	if(startTP!=TilePositions::None&&endTP!=TilePositions::None&&Broodwar->self()->hasResearched(TechTypes::Spider_Mines)){
		//!vultureGroup.isMining()&&
		if (this->lastMineStart!=startTP||this->lastMineEnd!=endTP){
			//Broodwar->printf("change mine route");
			vultureGroup.initMining(startTP,endTP);
			this->lastMineStart=startTP;
			this->lastMineEnd=endTP;
		}
		vultureGroup.doMining(1.5);
		//Broodwar->drawLineMap(startTP.x()*32+15, startTP.y()*32+15, endTP.x()*32+15, endTP.y()*32+15, Colors::Red);
	}
}
void StrategyReaction::MiningRouteInitialization()
{
	//get 3rd choke position
	TilePosition firstC = (TilePosition)this->mChokePoint->getFirstChoke()->getCenter();
	TilePosition thirdC = (TilePosition)this->mChokePoint->getThirdChoke()->getCenter();
	//if we already know enemy start location
	if (this->scm->enemyStartLocation){
		//if we have bunker postion
		if(this->gf->bunkerPosition){
			//start from bunker position
			startTP = TilePosition(this->gf->bunkerPosition->x()+1,this->gf->bunkerPosition->y()+1);
			endTP = this->scm->enemyStartLocation->getTilePosition();

		}
		//if we don't have bunker position
		else{
			startTP = TilePosition(this->mChokePoint->getFirstChoke()->getCenter().x()/32,this->mChokePoint->getFirstChoke()->getCenter().y()/32);
			//(TilePosition)this->mChokePoint->getSecondChoke()->getCenter();
			endTP = this->scm->enemyStartLocation->getTilePosition();			
		}
	}
	//if we still don't know where is enemy location
	else{
		//then take map center as end
		TilePosition mapCenter = TilePosition(Broodwar->mapWidth()/2,Broodwar->mapHeight()/2);
		//if we have bunker postion
		if(this->gf->bunkerPosition){
			startTP = TilePosition(this->gf->bunkerPosition->x()+1,this->gf->bunkerPosition->y()+1);
			endTP = mapCenter;
		}
		//if we don't have bunker position
		else{
			startTP = TilePosition(this->mChokePoint->getFirstChoke()->getCenter().x()/32,this->mChokePoint->getFirstChoke()->getCenter().y()/32);
			endTP = mapCenter;
		}
	}
}

void StrategyReaction::TvTTankProtectBase()
{
	UnitGroup tankGroup = SelectAll()(isCompleted)(Siege_Tank);
	//add to tank defend group

	for(std::map<Unit*,Position>::iterator i = TankDefBase.begin();i != TankDefBase.end();i++){
		if(this->allAttackUnits.find(i->first)!=this->allAttackUnits.end())
			this->allAttackUnits.erase(i->first);
	}


	//assign each tank a position to defend
	for each(BaseClass* bc in baseManager->getBaseSet()){
		//if this base already protected
		if(!bc->protectingArmy.empty())
			continue;
		else{
			for(std::map<Unit*,Position>::iterator i = TankDefBase.begin();i!=TankDefBase.end();i++){
				if (i->second!=Positions::None)
					continue;
				else{
					i->second = bc->getBaseLocation()->getPosition();
					bc->protectingArmy.insert(i->first);
					break;
				}
			}
		}
	}


	//go defend
	for(std::map<Unit*,Position>::iterator i = TankDefBase.begin();i != TankDefBase.end();i++){
		if (Broodwar->self()->hasResearched(TechTypes::Tank_Siege_Mode)){
			if (i->first->isSieged() && i->first->getPosition().getApproxDistance(i->second)<=32*3)
				continue;
			else{
				if (i->first->isSieged())
					i->first->unsiege();
				else{
					if (i->first->getPosition().getApproxDistance(i->second)<=32*3){
						i->first->siege();
						//Broodwar->printf("%s go to Pos(%d,%d)",i->first->getType().c_str(),i->second.x(),i->second.y());
					}

					else
						tankAttackMode(i->first,i->second);					
				}
			}				
		}				
		else
			return;				
	}	

}


void StrategyReaction::occupyPosition(Position p)
{	//check eTank num
	UnitGroup eTankInArea =SelectAllEnemy()(isVisible,canAttack).not(isWorker,isBuilding,isLifted).inRadius(32*12,p);
	if (eTankInArea(Siege_Tank).size()>allAttackUnits(Siege_Tank).size()){
		//Broodwar->printf("too much army there!");
	}
	else{
		//occupy the area
		//if eliminate all enemy in that area
		if (eTankInArea.empty()||!allAttackUnits.inRadius(32*12,p).empty()){
			occupyFlag=true;
		}
		else{
			for each(Unit* en in eTankInArea){
				allUnitAttack(en,en->getPosition());
			}
		}
	}
}

void StrategyReaction::keepFormationAttack(Unit* u){
	//Position safe = isUnderInvAtk(u);
	//if (safe!=Positions::None){
	//	if (u->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode){
	//		u->unsiege();
	//		return;
	//	}						
	//	else{
	//		u->move(safe);
	//		//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),safe.x(),safe.y(),Colors::Red);
	//		return;
	//	}					
	//}
	//else{
	Position gatherCenter = Positions::None;
	Position siegedTankCenter = this->allAttackUnits(Siege_Tank)(isSieged).getCenter();
	Position allTankCenter = this->allAttackUnits(Siege_Tank).getCenter();

	if (siegedTankCenter != Positions::None)
		gatherCenter = siegedTankCenter;
	else if(allTankCenter != Positions::None)
		gatherCenter = allTankCenter;
	else if(groupCenter)
		gatherCenter = *this->groupCenter;
	else
		gatherCenter = this->allAttackUnits.getCenter();

	if (u->getPosition().getApproxDistance(gatherCenter)>=32*8)
		u->attack(gatherCenter);
	else
	{
		Unit* bestT = getBestAttackTartget(u);
		if (bestT && (u->getGroundWeaponCooldown()==0))
		{
			u->attack(bestT);
			//Broodwar->printf("bestTar: %s",bestT->getType().c_str());
		}
		else
		{
			UnitGroup eG = SelectAllEnemy()(canAttack)(isDetected).inRadius(32*13,allAttackUnits.getCenter());
			if (!u->isAttacking())
				u->attack(eG.getCenter());
		} 
	}
	//}	
}



void StrategyReaction::BattlecruiserBehavior()
{
	//go to repair
	for each(Unit* b in battleC(isCompleted)(HitPoints,"<",250)){
		if(!b->isUnderAttack())
			continue;
		if (b->getPosition().getApproxDistance(this->mChokePoint->getSecondChoke()->getCenter())>32*4){
			if (!b->isMoving())
				b->move(this->mChokePoint->getSecondChoke()->getCenter());
		}
		else
			continue;
	}
	UnitGroup healthyBC = battleC(isCompleted)(HitPoints,">=",400);
	//first check the attack flag, if it is true
	if(this->mental->goAttack)
	{
		//then go to attack anyway
		for each(Unit* btc in healthyBC)
		{
			if (btc->isAttacking() && btc->getLastCommand().getType() == UnitCommandTypes::Attack_Move)
				continue;
			//yamato gun
			if (btc->getEnergy()>=150)
			{
				UnitGroup nearbyEnemy = SelectAllEnemy()(isCompleted)(canAttack)(isDetected)(HitPoints,">=",100);
				nearbyEnemy =	nearbyEnemy.not(isWorker).inRadius(btc->getType().sightRange(),btc->getPosition());
				MicroUnitControl::useYamatoGun(btc,nearbyEnemy,this->attTar.second);
			}
			else
				btc->attack(this->attTar.second);
		}
		return;
	}

	// if goAttack is false
	//check the healthy BC number
	if(healthyBC.size()<6)
	{
		if (!this->mental->enemyInSight(canAttack).empty())
		{
			for each(Unit* btc in battleC(isCompleted)(HitPoints,">=",400))
			{
				if (btc->isAttacking() && btc->getLastCommand().getType() == UnitCommandTypes::Attack_Move)
					continue;
				//yamato gun
				if (btc->getEnergy()>=150)
				{
					MicroUnitControl::useYamatoGun(btc,this->mental->enemyInSight,this->mental->enemyInSight(canAttack).getCenter());
				}
				else
					btc->attack(this->mental->enemyInSight(canAttack).getCenter());
			}
		}
		//if no enemy insight
		else
		{
			for each(Unit* btc in healthyBC)
				btc->attack(this->mChokePoint->mBestSetPoint);
		}
	}
	else
	{
		for each(Unit* btc in healthyBC)
		{
			if (btc->isAttacking() && btc->getLastCommand().getType() == UnitCommandTypes::Attack_Move)
				continue;
			//yamato gun
			if (btc->getEnergy()>=150)
			{
				UnitGroup nearbyEnemy = SelectAllEnemy()(isCompleted)(canAttack)(isDetected)(HitPoints,">=",100);
				nearbyEnemy =	nearbyEnemy.not(isWorker).inRadius(btc->getType().sightRange(),btc->getPosition());
				if (this->scm->enemyStartLocation)
					MicroUnitControl::useYamatoGun(btc,nearbyEnemy,this->scm->enemyStartLocation->getPosition());
				else
					MicroUnitControl::useYamatoGun(btc,nearbyEnemy,this->mChokePoint->mBestSetPoint);
			}
			else
			{
				if (this->scm->enemyStartLocation)
					btc->attack(this->scm->enemyStartLocation->getPosition());
				else
					btc->attack(this->mChokePoint->mBestSetPoint);
			}
		}
	}
}