#include "MicroUnitControl.h"

using namespace BWAPI;
using namespace BWTA;
using namespace ICEStarCraft;
using namespace std;

bool MicroUnitControl::isBlocked(Unit* u, Position p)
{
	if (!p.isValid() || !Broodwar->isWalkable(p.x() >> 3, p.y() >> 3))
	{
		return true;
	}

	Vector2 v = Vector2((double)(p.x() - u->getPosition().x()),(double)(p.y() - u->getPosition().y()));
	Position tmp;
	UnitType ut = u->getType();
	set<Unit*> units;
	set<Unit*>::iterator _u;

	for (int i = 1; i <= 4; i++)
	{
		tmp = v * 0.25 * i + u->getPosition();
		units = Broodwar->getUnitsInRectangle(tmp.x()-ut.dimensionLeft(),tmp.y()-ut.dimensionUp(),tmp.x()+ut.dimensionRight(),tmp.y()+ut.dimensionDown());
		
		_u = units.find(u);
		if (_u != units.end())
			units.erase(_u);

		if (!units.empty())
			return true;
	}

	return false;
}

UnitGroup MicroUnitControl::getBlockingUnits(Unit *u, Position p)
{
	Vector2 v = Vector2((double)(p.x() - u->getPosition().x()),(double)(p.y() - u->getPosition().y()));
	Position tmp;
	UnitType ut = u->getType();
	UnitGroup units;

	for (int i = 1; i <= 4; i++)
	{
		tmp = v * 0.25 * i + u->getPosition();
		for each (Unit* t in Broodwar->getUnitsInRectangle(tmp.x()-ut.dimensionLeft(),tmp.y()-ut.dimensionUp(),tmp.x()+ut.dimensionRight(),tmp.y()+ut.dimensionDown()))
			units.insert(t);
	}	
	
	set<Unit*>::iterator _u = units.find(u);
	if (_u != units.end())
		units.erase(_u);

	return units;
}

Position MicroUnitControl::getDestination(Unit*u, UnitGroup myUnits, UnitGroup enemyUnits)
{
	Position destination;
	Vector2 v, v1, v2;

	v1 = enemyUnits.size() > 1 ? Vector2(myUnits.getCenter()) : Vector2(u->getPosition());
	v2 = Vector2(enemyUnits.getCenter());

	//uniform flow
	v = v2 - v1;
	if (v.approxLen() > 32*8)
	{
		v = v * (32*8.0 / v.approxLen());
	}
	
	for each (Unit* e in enemyUnits)
	{
		if (e->getPosition().getApproxDistance(u->getPosition()) <= u->getType().sightRange())
			v += PFFunctions::getVelocitySource(e->getPosition(), u->getPosition()) * 1000;
	}

	v = v * (96.0 / v.approxLen());
	destination = v + u->getPosition();

	if (!isBlocked(u, destination))
	{
		return destination;
	}

	Position p = u->getPosition();
	UnitType ut = u->getType();
	//Broodwar->drawBoxMap(p.x()-ut.dimensionLeft(),p.y()-ut.dimensionUp(),p.x()+ut.dimensionRight(),p.y()+ut.dimensionDown(),Colors::Red);

	Vector2 _v = Vector2();
	int left  = 0;
	int right = 0;
	int r = 64;

	set<Unit*> surroundUnits;;
	for each (Unit* i in u->getUnitsInRadius(r))
	{
		if (i->getPosition().getApproxDistance(enemyUnits.getCenter()) - 32 < u->getPosition().getApproxDistance(enemyUnits.getCenter()))
		{
			if (i->getPlayer() == Broodwar->self() && (i->isAttacking() || i->getType() == UnitTypes::Terran_Siege_Tank_Tank_Mode || i->getType() == UnitTypes::Terran_Siege_Tank_Siege_Mode))
			{		
				surroundUnits.insert(i);
			}
			if (i->getType().isInvincible() || (i->getType().isBuilding() && !i->isLifted()) || i->getType().isAddon())
			{
				surroundUnits.insert(i);
			}
		}
	}

	if (surroundUnits.empty())
	{
		return destination;
	}

	for each (Unit* i in surroundUnits)
	{
		Vector2 v1 = v;
		Vector2 v2 = Vector2(i->getPosition().x() - u->getPosition().x(), i->getPosition().y() - u->getPosition().y());

		if (v1.x() * v2.y() - v2.x() * v1.y() > 0)
			left++;
		else
			right++;

		//Broodwar->drawLineMap(u->getPosition().x(),u->getPosition().y(),i->getPosition().x(),i->getPosition().y(),Colors::Orange);
		_v += PFFunctions::getVelocityVortex(i->getPosition(),u->getPosition()) * 1000;
	}

	_v = _v * (96.0 / _v.approxLen());
	Position p1 = -_v + u->getPosition();
	Position p2 = _v + u->getPosition();
	
	if(ICEStarCraft::Helper::isWalkable(p,p1) && ICEStarCraft::Helper::isWalkable(p,p2))
	{
		return left > right ? p1 : p2;
	}

	if(ICEStarCraft::Helper::isWalkable(p,p1) && !ICEStarCraft::Helper::isWalkable(p,p2))
	{
		return p1;
	}

	if(!ICEStarCraft::Helper::isWalkable(p,p1) && ICEStarCraft::Helper::isWalkable(p,p2))
	{
		return p2;
	}

	Broodwar->drawTextMap(u->getPosition().x(),u->getPosition().y(),"nowhere to go");
	return Positions::None;
}

Position MicroUnitControl::getDestination2(Unit*u, UnitGroup myUnits, Position atkTar, BWTA::Chokepoint* theChokePoint, ChokepointManager* mChokePoint)
{
	Position p = u->getPosition();
	BWTA::Region* reg = BWTA::getRegion(p);
	BWTA::Region* targetReg = BWTA::getRegion(atkTar);

	if (!reg || !targetReg)
	{
		return Positions::None;
	}

	if (u->getType() == UnitTypes::Terran_Siege_Tank_Siege_Mode && p.getApproxDistance(theChokePoint->getCenter()) < 32*3)
	{
		u->unsiege();
		return Positions::None;
	}
	
	if (u->isAttacking() || u->isAttackFrame() || u->getLastCommand().getType() == UnitCommandTypes::Attack_Unit || u->getOrder() == Orders::AttackUnit)
	{
		if (p.getApproxDistance(theChokePoint->getCenter()) > 32*3)
		{
			return Positions::None;
		}
	}
	
	if (p.getApproxDistance(theChokePoint->getCenter()) > 32*6)
	{
		return Positions::None;
	}

	if (reg != targetReg && !ICEStarCraft::Helper::isConnected(reg,targetReg))
	{
		return Positions::None;
	}

	UnitGroup insideUnits = myUnits.inRegion(targetReg);
	double radius = 32*4*(1.5-0.8*insideUnits.size()/myUnits.size());
	//Broodwar->drawCircleMap(theChokePoint->getCenter().x(),theChokePoint->getCenter().y(),(int)radius,Colors::Red);
	if (reg == targetReg && p.getApproxDistance(theChokePoint->getCenter()) > radius)
	{
		return Positions::None;
	}

	Vector2 v1 = theChokePoint->getSides().first - theChokePoint->getSides().second;
	Vector2 v2 = Vector2(v1.y(),-v1.x());
	Position regcen = v2 + theChokePoint->getCenter();
	if (BWTA::getRegion(regcen) != targetReg)
	{
		regcen = -v2 + theChokePoint->getCenter();
	}
	//Broodwar->drawCircleMap(regcen.x(),regcen.y(),5,Colors::Black,true);


	Vector2 b = Vector2();
	Vector2 b_tmp = Vector2();
	set<Position> vertices = mChokePoint->regionVertices[reg];

	
	for (set<Position>::iterator i = vertices.begin(); i != vertices.end(); i++)
	{
		Broodwar->drawCircleMap((*i).x(),(*i).y(),2,Colors::Yellow);
		if (!ICEStarCraft::Helper::isChokePointCenter(*i) && p.getApproxDistance(*i) <= 64)
		{
			//Broodwar->drawLineMap((*i).x(),(*i).y(),p.x(),p.y(),Colors::Yellow);
			b_tmp = PFFunctions::getVelocityVortex(*i,p)*1000;

			v1 = (*i) - regcen;
			v2 = theChokePoint->getCenter() - regcen;
			if(v1.x()*v2.y() - v1.y()*v2.x() < 0)
				b += b_tmp;
			else
				b -= b_tmp;
		}
	}
	if (b == Vector2(0,0))
	{
		return Positions::None;
	}
	b = b * (96.0 / b.approxLen());
	return b + p;
}

void MicroUnitControl::useYamatoGun(Unit* btc, UnitGroup& enemy, Position atkPosition)
{
	if (btc->getEnergy() < 150)
		return;

	if (btc->getLastCommand().getType() == UnitCommandTypes::Use_Tech)
		return;

	Unit* bestYMT = NULL;
	int dis = 99999;
	
	for each (Unit* e in enemy)
	{
		if (btc->getPosition().getApproxDistance(e->getPosition())<dis)
		{
			dis = btc->getPosition().getApproxDistance(e->getPosition());
			bestYMT = e;
		}
	}
	
	if(bestYMT)
	{
		btc->useTech(TechTypes::Yamato_Gun,bestYMT);
	}
	else
	{
		btc->attack(atkPosition);
	}	
}