#include "PFFunctions.h"

using namespace BWAPI;
using namespace BWTA;
using namespace ICEStarCraft;
using namespace std;

/* Calculate velocity of a source */
Vector2 PFFunctions::getVelocitySource(Position& s, Position& p)
{
	int x = p.x() - s.x();
	int y = p.y() - s.y();
	double r2 = 1.0*x*x + 1.0*y*y;

	return Vector2(x/r2, y/r2);
}

/* Calculate velocity of a vortex */
Vector2 PFFunctions::getVelocityVortex(Position& s, Position& p)
{
	int x = p.x() - s.x();
	int y = p.y() - s.y();
	double r2 =  1.0*x*x +  1.0*y*y;

	return Vector2(y/r2, -x/r2);
}

/*
 * obstacle in uniform flow
 * o: obstacle's position
 * p: unit's position
 * r: obstacle's radius
 * c: cos alpha 
 * s: sin alpha
 */
Vector2 PFFunctions::getVelocityObstacle(const Position& o, const Position& p, const double r, const double c, const double s)
{
	int x = p.x() - o.x();
	int y = p.y() - o.y();
	double x2 = 1.0*x*x;
	double y2 = 1.0*y*y;
	double r2 = r*r;
	double r4 = (x2 + y2)*(x2 + y2);

	double vx = r2 * (c*(y2-x2) - 2*s*x*y) / r4;
	double vy = r2 * (s*(x2-y2) - 2*c*x*y) / r4;
	return Vector2(vx,vy);
}

/* Calculate velocity of potential flow created by border */
Vector2 PFFunctions::getVelocityBorder(Position& p)
{
	if (!p.isValid())
	{
		return Vector2();
	}

	BWTA::Region* reg = BWTA::getRegion(p);
	Polygon border = reg->getPolygon();
	Vector2 b = Vector2();

	for (Polygon::iterator i = border.begin(); i != border.end(); i++)
	{
		if (p.getApproxDistance(*i) <= 32)
		{
			Broodwar->drawCircleMap((*i).x(),(*i).y(),2,Colors::Yellow);
			b += getVelocitySource(*i,p);
		}
	}

	return b;
}