#include "ExampleAIModule.h"
#include <iostream>
#include <fstream>

using namespace BWAPI;
using namespace ICEStarCraft;

bool analyzed;
BWTA::Region* home;
BWTA::Region* enemy_base;

void ExampleAIModule::onStart()
{
	if (Broodwar->isReplay()) return;

#ifdef _LOG_TO_FILE
	ifstream _in1("retryTimesMax.txt", ios::in);
	if (_in1.is_open())
	{
		_in1 >> reTryTimesMax;
		_in1.close();
	}
	else
	{
		reTryTimesMax = 50;
	}	
	
	reTryTimes = -1;
	ifstream _in("temp.txt", ios::in);
	if (_in.is_open())
	{
		_in >> reTryTimes;
		_in.close();
	}
	
	if(reTryTimes == -1)
	{
		ofstream _out("ICEStarCraft_log.csv", ios::app);
		_out << Broodwar->mapFileName().c_str() << endl;
		_out << "Win/Lose,FrameCount,UnitScore,KillScore,BuildingScore,RazingScore" << endl;
		_out.close();
		reTryTimes = reTryTimesMax;
	}

	reTryTimes--;
	fstream _out("temp.txt", ios::out);
	_out << reTryTimes;
	_out.close();
	if (reTryTimes < 0)
	{
		Broodwar->pauseGame();
		//return;
	}
	else
	{
		Broodwar->printf("%d games left...",reTryTimes+1);
	}
#endif

	BWTA::readMap();
	BWTA::analyze();
	analyzed=true;
	Broodwar->printf("Hi, This is ICEbot!");
	Broodwar->printf("GL && GG warning LNK4075: ignoring '/INCREMENTAL' due to '/LTCG' specification");
  //Broodwar->printf("%s", Broodwar->mapFileName().c_str());

	//Broodwar->enableFlag(Flag::UserInput);
	Broodwar->setLocalSpeed(0);

	unitGroupManager = UnitGroupManager::create();
	workerManager = WorkerManager::create();
	this->upgradeManager     = new UpgradeManager(&this->arbitrator);
	this->techManager        = new TechManager(&this->arbitrator);
	this->buildManager       = new BuildManager(&this->arbitrator);
	this->supplyManager      = SupplyManager::create();
	this->scoutManager	   = ScoutManager::create();
	this->strategyReact = StrategyReaction::create();

	this->buildOrderManager  = new BuildOrderManager(this->buildManager,this->techManager,this->upgradeManager,workerManager,this->supplyManager);
	this->buildOrderManager->setDebugMode(false);
	this->chokePointManager = ChokepointManager::create();
	this->gameflow = GameFlow::create();
	this->mental = MentalClass::create();
	this->supplyManager->setBuildManager(this->buildManager);
	this->supplyManager->setBuildOrderManager(this->buildOrderManager);
	this->techManager->setBuildingPlacer(this->buildManager->getBuildingPlacer());
	this->upgradeManager->setBuildingPlacer(this->buildManager->getBuildingPlacer());

	this->buildOrderManager->enableDependencyResolver();
	this->chokePointManager->setUsefulChokepoint();
	this->chokePointManager->getRegionVertices();
	this->myInfoManager = MyInforManager::create();
	this->enemyInfoM= EnemyInforManager::create();
	//attackpoint = BWTA::getRegion(Broodwar->enemy()->getStartLocation())->getCenter();

	//productionManager = new ProductionManager();
	baseManager = BaseManager::create();
	baseManager->expandPlan();
	this->myInfoManager->setUsefulManagers(baseManager);
	this->buildOrderManager->setBaseManagerClass(baseManager);
	workerManager->setBaseManagerClass(baseManager);
	workerManager->setBuildOrderManager(this->buildOrderManager);
	baseManager->setManagers(buildOrderManager);
	this->scoutManager->setManagers();
	this->chokePointManager->setScoutManager(scoutManager);
	this->mental->setManagers(this->buildOrderManager,upgradeManager,techManager);
	this->tpTiming = TPTiming::create();

	/********autobuild worker********/

	workerManager->enableAutoBuild();
	workerManager->setAutoBuildPriority(70);



	this->gameflow->setManagers(this->buildOrderManager,this->upgradeManager);

	BWAPI::Race race = Broodwar->self()->getRace();
	this->simulator = new Simulator();

	strategyReact->setUsefulManagers(buildOrderManager);

	terrainGraph = TerrainGraph::create();

	/*if (race == Races::Terran){
		this->gameflow->TPstart();
	}*/
	extest=1;
	this->buildOrderManager->buildAdditional(2,UnitTypes::Terran_SCV,300);
	this->drawObjects = false;
	draw_self_targets = false;
	draw_enemy_targets = false;
}

void ExampleAIModule::onEnd(bool isWinner)
{
#ifdef _LOG_TO_FILE
	if (reTryTimes >= 0)
	{
		//_out << "Win/Lose,FrameCount,UnitScore,KillScore,BuildingScore,RazingScore" << endl;
		ofstream _out("ICEStarCraft_log.csv", ios::app);
		string result = Broodwar->self()->isVictorious() ? "win" : "lose";
		_out << result.c_str() << ",";
		_out << Broodwar->getFrameCount() << ",";
		_out << Broodwar->self()->getUnitScore() << ",";
		_out << Broodwar->self()->getKillScore() << ",";
		_out << Broodwar->self()->getBuildingScore() << ",";
		_out << Broodwar->self()->getRazingScore() << ",";
		_out << endl;
		_out.close();
	}
#endif

	MentalClass::destroy();
	WorkerManager::destroy();
	ScoutManager::destroy();
	UnitGroupManager::destroy();
	ChokepointManager::destroy();
	StrategyReaction::destroy();
	MyInforManager::destroy();
	EnemyInforManager::destroy();
	GameFlow::destroy();
	BaseManager::destroy();
	TPTiming::destroy();
	if (isWinner){
		//log win to file
	}

#ifdef _LOG_TO_FILE
  ofstream fout("result.csv", ios::app);
  if (fout.is_open()) {
    fout << Broodwar->self()->getRace().c_str() << "(" << Broodwar->self()->getName() << ")"  << ",";
    fout << Broodwar->enemy()->getRace().c_str() << "(" << Broodwar->enemy()->getName() << ")"  << ",";
    fout << isWinner << endl;
  } else cout << "FU";
#endif
}

void ExampleAIModule::onFrame()
{
	if (Broodwar->isReplay()) return;
	else
	{
		if (this->drawObjects){
			enhancedUI->update();
			enhancedUI->drawUnitsAttackRange();
			terrainGraph->draw();
		}

		if (draw_enemy_targets) enhancedUI->drawUnitTargets(Broodwar->enemy());
		if (draw_self_targets) enhancedUI->drawUnitTargets(Broodwar->self());
			
		//this->chokePointManager->onFrame();
		this->gameflow->TZonFrame();


/*		if (Broodwar->enemy()->getRace()==Races::Protoss)
			this->gameflow->balanceArmyCombination();*/	
		this->strategyReact->ScanInvisibleEnemy();
		this->mental->setEnemyPlanFlag();
		this->mental->counterMeasure();
		this->mental->upDateSightRange();
		this->mental->attackTimingCheck();
		if (Broodwar->getFrameCount()%24 == 0)
			this->mental->baseUnderAttack();
		strategyReact->normalTVPDefend();
		if (Broodwar->enemy()->getRace() == Races::Terran){
			strategyReact->TvTTankProtectBase();
		}
			

		strategyReact->showScanPositionTable();
		this->enemyInfoM->showTypeToTimeMap();
		this->enemyInfoM->showUnitToTypeMap();
		this->enemyInfoM->showBuildingToPositionMap();
		this->enemyInfoM->showBaseToDataMap();
		this->enemyInfoM->onFrame();

		this->myInfoManager->myInfoOnFrame();
		this->tpTiming->CheckTiming();

		//this->scoutManager->SCVScout(ScoutManager::ScoutPurpose::EnemyExpansion);
		//strategyReact->ChokePointBlockSolution();
		/*	if (Broodwar->getFrameCount()>24*60*6 && Broodwar->getFrameCount()<24*60*8)		
		strategyReact->defendDK();*/

		this->scoutManager->onFrame();
		this->chokePointManager->getEnemyChokePoints();
		

		workerManager->onFrame();

		baseManager->update();
		this->buildManager->update();
		this->buildOrderManager->update();
		this->supplyManager->update();
		this->arbitrator.update();
		this->upgradeManager->update();
		this->techManager->update();

#ifdef _BATTLE_DEBUG
		/*for (int i = 0; i < Broodwar->mapWidth(); i++)
		{
			for (int j = 0; j < Broodwar->mapHeight(); j++)
			{
				Broodwar->drawTextMap(32*i+16,32*j+16,"%d-%d",i,j);
			}	
		}
		for each(BWTA::BaseLocation* b in BWTA::getBaseLocations())
		{
			Broodwar->drawCircleMap(b->getPosition().x(),b->getPosition().y(),5,Colors::Blue);
		}*/
		

		for each (TilePosition tp in this->chokePointManager->dropReg)
		{
			Broodwar->drawTextMap(tp.x()*32+16,tp.y()*32+16,"x");
		}
		if (this->strategyReact->dropPos != TilePositions::None)
		{
			Broodwar->drawCircleMap(this->strategyReact->dropPos.x()*32+16,this->strategyReact->dropPos.y()*32+16,10,Colors::Green);
			Broodwar->drawCircleMap(this->strategyReact->dropPos.x()*32+16,this->strategyReact->dropPos.y()*32+16,20,Colors::Green);
		}
		
		/*for each (BWTA::Region* r in BWTA::getRegions())
		{
			for each (Position p in ChokepointManager::create()->regionVertices[r])
				Broodwar->drawCircleMap(p.x(),p.y(), 5, Colors::Yellow);
		}*/
		// print allAttackUnits info
		map<UnitType,int> list;
		int num;
		int line = 0;
		for each (Unit* u in this->strategyReact->allAttackUnits)
		{
			Broodwar->drawCircleMap(u->getPosition().x(),u->getPosition().y(),2,Colors::Green,true);
			if (list.empty())
			{
				list.insert(make_pair(u->getType(),1));
			}
			else
			{
				map<UnitType,int>::iterator i = list.find(u->getType());
				if (i != list.end())
				{
					num = (*i).second;
					list.erase(i);
					list.insert(make_pair(u->getType(),num+1));
				}
				else
				{
					list.insert(make_pair(u->getType(),1));
				}
			}
		}
		Broodwar->drawTextScreen(450,line*10+60,"allAttackUnits: %d/%d",this->strategyReact->myBattleUnits.size(),this->strategyReact->allAttackUnits.size());
		line++;
		for (map<UnitType,int>::iterator i = list.begin(); i != list.end(); i++)
		{
			Broodwar->drawTextScreen(450,line*10+60,"%d %s",(*i).second,(*i).first.getName().c_str());
			line++;
		}
	
		// print enemyInSight info
		list.clear();
		line++;
		for each (Unit* u in this->mental->enemyInSight)
		{
			Broodwar->drawCircleMap(u->getPosition().x(),u->getPosition().y(),2,Colors::Red,true);
			if (list.empty())
			{
				list.insert(make_pair(u->getType(),1));
			}
			else
			{
				map<UnitType,int>::iterator i = list.find(u->getType());
				if (i != list.end())
				{
					num = (*i).second;
					list.erase(i);
					list.insert(make_pair(u->getType(),num+1));
				}
				else
				{
					list.insert(make_pair(u->getType(),1));
				}
			}
		}
		Broodwar->drawTextScreen(450,line*10+60,"enemyInSight: %d",this->mental->enemyInSight.size());
		line++;
		for (map<UnitType,int>::iterator i = list.begin(); i != list.end(); i++)
		{
			Broodwar->drawTextScreen(450,line*10+60,"%d %s",(*i).second,(*i).first.getName().c_str());
			line++;
		}
		list.clear();

		// draw circle around units that are taking part in the battle
		for each (Unit* u in this->strategyReact->myBattleUnits)
		{
			//int r = (u->getType().dimensionLeft() + u->getType().dimensionUp() + u->getType().dimensionRight() + u->getType().dimensionDown()) / 2;
			Broodwar->drawCircleMap(u->getPosition().x(),u->getPosition().y(),10,Colors::Green);
		}

		for each (Unit* u in this->strategyReact->enemyBattleUnits)
		{
			//int r = (u->getType().dimensionLeft() + u->getType().dimensionUp() + u->getType().dimensionRight() + u->getType().dimensionDown()) / 2;
			Broodwar->drawCircleMap(u->getPosition().x(),u->getPosition().y(),10,Colors::Red);
		}

		// draw center of myBattleUnits
		Position cen = this->strategyReact->myBattleUnits.getCenter();
		Broodwar->drawCircleMap(cen.x(),cen.y(),3,Colors::Orange,true);
		Broodwar->drawCircleMap(cen.x(),cen.y(),8,Colors::Orange);
		//this->strategyReact->myBattleUnits.clear();

		// draw center of enemyBattleUnits
		cen = this->strategyReact->enemyBattleUnits.getCenter();
		Broodwar->drawCircleMap(cen.x(),cen.y(),3,Colors::Red,true);
		Broodwar->drawCircleMap(cen.x(),cen.y(),8,Colors::Red);

		// draw choke points
		for each (BWTA::Chokepoint* cp in BWTA::getChokepoints())
		{
			Broodwar->drawCircleMap(cp->getCenter().x(),cp->getCenter().y(),5,Colors::Brown,true);
			Broodwar->drawTextMap(cp->getCenter().x()+5,cp->getCenter().y(),"%d",(int)cp->getWidth());
		}

		// draw region which we attack
		//Broodwar->printf("%s %s",this->strategyReact->attTar.first.c_str(),this->strategyReact->attTar.first != "No_AtkTar"?"true":"false");
		if (this->strategyReact->attTar.first != "No_AtkTar")
		{
			BWTA::Region* r = BWTA::getRegion(this->strategyReact->attTar.second);
			if (r)
			{
				BWTA::Polygon p = r->getPolygon();
				set<Chokepoint*> cp = BWTA::getChokepoints();
				bool isChokePoint = false;

				for (Polygon::iterator i = p.begin(); i != p.end(); i++)
				{
					isChokePoint = false;
					for (set<Chokepoint*>::iterator c = cp.begin(); c != cp.end(); c++)
					{
						if ((*i) == (*c)->getCenter())
						{
							isChokePoint = true;
							break;
						}
					}
					Broodwar->drawCircleMap((*i).x(),(*i).y(),2,Colors::Yellow,isChokePoint);
				}
			}
		}

#endif // _BATTLE_DEBUG
#ifdef _LOG_TO_FILE
		Broodwar->drawTextScreen(0,50,"%d games left",reTryTimes+1);
#endif // _LOG_TO_FILE
	}
}

void ExampleAIModule::onSendText(std::string text)
{
	if (text=="/drawoff"){
		this->drawObjects = false;
	}
	if (text=="/drawon"){
		this->drawObjects = true;
	}

	if (text == "/e"){
		draw_enemy_targets = !draw_enemy_targets;
	}
	if (text == "/o"){
		draw_self_targets = !draw_self_targets;
	}

	if (text=="/a"){
		Broodwar->sendText("show me the money");
		Broodwar->sendText("operation cwal");
		Broodwar->sendText("/speed-");
		Broodwar->sendText("food for thought");
		Broodwar->sendText("power overwhelming");
	}
	if (text=="/wudi"){
		Broodwar->sendText("power overwhelming");
	}
	if (text=="/tt"){
		this->enemyInfoM->showTypeToTime=true;
		this->enemyInfoM->showUnitToType=false;
		this->enemyInfoM->showBuildingToPosition=false;
		this->enemyInfoM->showBaseToData=false;
	}
	if (text=="/ut"){
		this->enemyInfoM->showTypeToTime=false;
		this->enemyInfoM->showUnitToType=true;
		this->enemyInfoM->showBuildingToPosition=false;
		this->enemyInfoM->showBaseToData=false;
	}
	if (text=="/bp"){
		this->enemyInfoM->showTypeToTime=false;
		this->enemyInfoM->showUnitToType=false;
		this->enemyInfoM->showBuildingToPosition=true;
		this->enemyInfoM->showBaseToData=false;
	}

	if (text=="/bd"){
		this->enemyInfoM->showTypeToTime=false;
		this->enemyInfoM->showUnitToType=false;
		this->enemyInfoM->showBuildingToPosition=false;
		this->enemyInfoM->showBaseToData=true;
	}
	if (text=="/tableoff"){
		this->enemyInfoM->showTypeToTime=false;
		this->enemyInfoM->showUnitToType=false;
		this->enemyInfoM->showBuildingToPosition=false;
		this->enemyInfoM->showBaseToData=false;
	}
	if (text=="/ston"){
		this->strategyReact->setScanTableFlag(true);
	}
	if (text=="/stoff"){
		this->strategyReact->setScanTableFlag(false);
	}

	if (text=="/c")
	{
		for each(BaseClass* bc in baseManager->getBaseSet())
		{
			Broodwar->printf("need:%d, current:%d\n",bc->getNeedWorkerNum(),bc->getCurrentWorkerNum());
		}	  
	}

	if (text == "/simu"){
		simulator->ProtossSimu(Broodwar->getFrameCount(),pEconomic,50);
	}
	if (text =="/dk")
	{
		strategyReact->defendDK();
	}

	if (text =="/ex")
	{
		buildOrderManager->autoExpand(40,++extest);
	}
	if (text=="/g3"){
		workerManager->setNeedGasLevel(3);
	}
	if (text=="/g2"){
		workerManager->setNeedGasLevel(2);
	}
	if (text=="/g1"){
		workerManager->setNeedGasLevel(1);
	}
	if (text=="/g0"){
		workerManager->setNeedGasLevel(0);
	}

	if (text=="/autoscv"){
		Broodwar->printf("enable auto build scv");
		workerManager->enableAutoBuild();
	}

	if (text=="/stopscv"){
		Broodwar->printf("disable auto build scv");
		workerManager->disableAutoBuild();

	}



	if (text=="/show bullets")
	{
		show_bullets = !show_bullets;
	} else if (text=="/show players")
	{
		showPlayers();
	} else if (text=="/show forces")
	{
		showForces();
	} else if (text=="/show visibility")
	{
		show_visibility_data=!show_visibility_data;
	} else if (text=="/analyze")
	{
		if (analyzed == false)
		{
			Broodwar->printf("Analyzing map... this may take a minute");
			CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)AnalyzeThread, NULL, 0, NULL);
		}
	} else
	{
		//  Broodwar->printf("You typed '%s'!",text.c_str());
		Broodwar->sendText("%s",text.c_str());
	}
}

void ExampleAIModule::onReceiveText(BWAPI::Player* player, std::string text)
{
	Broodwar->printf("%s said '%s'", player->getName().c_str(), text.c_str());
}

void ExampleAIModule::onPlayerLeft(BWAPI::Player* player)
{
	Broodwar->sendText("%s left the game.",player->getName().c_str());
}

void ExampleAIModule::onNukeDetect(BWAPI::Position target)
{
	if (target!=Positions::Unknown)
		Broodwar->printf("Nuclear Launch Detected at (%d,%d)",target.x(),target.y());
	else
		Broodwar->printf("Nuclear Launch Detected");
}

void ExampleAIModule::onUnitDiscover(BWAPI::Unit* unit)
{
	if (Broodwar->isReplay())return;
	/*if (unit->getPlayer() == Broodwar->self())
	{
		Broodwar->printf("onUnitDiscover: %s",unit->getType().getName().c_str());
	}*/
	
	this->myInfoManager->onUnitDiscover(unit);
	this->enemyInfoM->onUnitDiscover(unit);
	unitGroupManager->onUnitDiscover(unit);
	this->strategyReact->onUnitDiscover(unit);
	workerManager->onUnitDiscover(unit);
	//this->scoutManager->OnUnitDiscover(unit);
}

void ExampleAIModule::onUnitEvade(BWAPI::Unit* unit)
{
	unitGroupManager->onUnitEvade(unit);
	this->mental->onUnitEvade(unit);
	if (Broodwar->self()->isEnemy(unit->getPlayer())&& unit->getRegion()->isHigherGround()&&
		(unit->getType()==UnitTypes::Terran_Siege_Tank_Siege_Mode || unit->getType()==UnitTypes::Protoss_Reaver || unit->getType()==UnitTypes::Protoss_High_Templar)){
		UnitGroup allMyScanner = SelectAll()(isCompleted)(Comsat_Station)(Energy,">=",50);
		if (!allMyScanner.empty()){
			BWAPI::Unit* scanner = (*allMyScanner.begin());
			scanner->useTech(TechTypes::Scanner_Sweep,unit->getPosition());
		}
	}
}

void ExampleAIModule::onUnitShow(BWAPI::Unit* unit)
{
	this->enemyInfoM->onUnitDiscover(unit);
}

void ExampleAIModule::onUnitHide(BWAPI::Unit* unit)
{
	workerManager->onUnitHide(unit);
}

void ExampleAIModule::onUnitCreate(BWAPI::Unit* unit)
{

}

void ExampleAIModule::onUnitDestroy(BWAPI::Unit* unit)
{
	this->mental->onUnitDestroy(unit);
	workerManager->onUnitDestroy(unit);
	this->strategyReact->onUnitDestroy(unit);
	this->buildOrderManager->onUnitDestroy(unit);
	this->enemyInfoM->onUnitDestroy(unit);
	this->myInfoManager->onUnitDestroy(unit);
	this->scoutManager->onUnitDestroy(unit);
	this->arbitrator.onRemoveObject(unit);
	this->buildManager->onRemoveUnit(unit);
	this->upgradeManager->onRemoveUnit(unit);
	this->techManager->onRemoveUnit(unit);
	//this->strategyReact->onUnitDestroy(unit);
	baseManager->onUnitDestroy(unit);
}

void ExampleAIModule::onUnitMorph(BWAPI::Unit* unit)
{
	if (!Broodwar->isReplay()){
		this->mental->onUnitMorph(unit);
		this->enemyInfoM->onUnitMorph(unit);
		workerManager->onUnitMorph(unit);
	}
		
}

void ExampleAIModule::onUnitRenegade(BWAPI::Unit* unit)
{
	unitGroupManager->onUnitRenegade(unit);
	if (!Broodwar->isReplay()) return;
	// Broodwar->sendText("A %s [%x] is now owned by %s",unit->getType().getName().c_str(),unit,unit->getPlayer()->getName().c_str());
}

void ExampleAIModule::onSaveGame(std::string gameName)
{
	Broodwar->printf("The game was saved to \"%s\".", gameName.c_str());
}

DWORD WINAPI AnalyzeThread()
{
	BWTA::analyze();

	//self start location only available if the map has base locations
	if (BWTA::getStartLocation(BWAPI::Broodwar->self())!=NULL)
	{
		home       = BWTA::getStartLocation(BWAPI::Broodwar->self())->getRegion();
	}
	//enemy start location only available if Complete Map Information is enabled.
	if (BWTA::getStartLocation(BWAPI::Broodwar->enemy())!=NULL)
	{
		enemy_base = BWTA::getStartLocation(BWAPI::Broodwar->enemy())->getRegion();
	}
	analyzed   = true;
	return 0;
}

void ExampleAIModule::drawStats()
{
	std::set<Unit*> myUnits = Broodwar->self()->getUnits();
	Broodwar->drawTextScreen(5,0,"I have %d units:",myUnits.size());
	std::map<UnitType, int> unitTypeCounts;
	for(std::set<Unit*>::iterator i=myUnits.begin();i!=myUnits.end();i++)
	{
		if (unitTypeCounts.find((*i)->getType())==unitTypeCounts.end())
		{
			unitTypeCounts.insert(std::make_pair((*i)->getType(),0));
		}
		unitTypeCounts.find((*i)->getType())->second++;
	}
	int line=1;
	for(std::map<UnitType,int>::iterator i=unitTypeCounts.begin();i!=unitTypeCounts.end();i++)
	{
		Broodwar->drawTextScreen(5,16*line,"- %d %ss",(*i).second, (*i).first.getName().c_str());
		line++;
	}
}

void ExampleAIModule::drawBullets()
{
	std::set<Bullet*> bullets = Broodwar->getBullets();
	for(std::set<Bullet*>::iterator i=bullets.begin();i!=bullets.end();i++)
	{
		Position p=(*i)->getPosition();
		double velocityX = (*i)->getVelocityX();
		double velocityY = (*i)->getVelocityY();
		if ((*i)->getPlayer()==Broodwar->self())
		{
			Broodwar->drawLineMap(p.x(),p.y(),p.x()+(int)velocityX,p.y()+(int)velocityY,Colors::Green);
			Broodwar->drawTextMap(p.x(),p.y(),"\x07%s",(*i)->getType().getName().c_str());
		}
		else
		{
			Broodwar->drawLineMap(p.x(),p.y(),p.x()+(int)velocityX,p.y()+(int)velocityY,Colors::Red);
			Broodwar->drawTextMap(p.x(),p.y(),"\x06%s",(*i)->getType().getName().c_str());
		}
	}
}

void ExampleAIModule::drawVisibilityData()
{
	for(int x=0;x<Broodwar->mapWidth();x++)
	{
		for(int y=0;y<Broodwar->mapHeight();y++)
		{
			if (Broodwar->isExplored(x,y))
			{
				if (Broodwar->isVisible(x,y))
					Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Green);
				else
					Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Blue);
			}
			else
				Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Red);
		}
	}
}

void ExampleAIModule::drawTerrainData()
{
	//we will iterate through all the base locations, and draw their outlines.
	for(std::set<BWTA::BaseLocation*>::const_iterator i=BWTA::getBaseLocations().begin();i!=BWTA::getBaseLocations().end();i++)
	{
		TilePosition p=(*i)->getTilePosition();
		Position c=(*i)->getPosition();

		//draw outline of center location
		Broodwar->drawBox(CoordinateType::Map,p.x()*32,p.y()*32,p.x()*32+4*32,p.y()*32+3*32,Colors::Blue,false);

		//draw a circle at each mineral patch
		for(std::set<BWAPI::Unit*>::const_iterator j=(*i)->getStaticMinerals().begin();j!=(*i)->getStaticMinerals().end();j++)
		{
			Position q=(*j)->getInitialPosition();
			Broodwar->drawCircle(CoordinateType::Map,q.x(),q.y(),30,Colors::Cyan,false);
		}

		//draw the outlines of vespene geysers
		for(std::set<BWAPI::Unit*>::const_iterator j=(*i)->getGeysers().begin();j!=(*i)->getGeysers().end();j++)
		{
			TilePosition q=(*j)->getInitialTilePosition();
			Broodwar->drawBox(CoordinateType::Map,q.x()*32,q.y()*32,q.x()*32+4*32,q.y()*32+2*32,Colors::Orange,false);
		}

		//if this is an island expansion, draw a yellow circle around the base location
		if ((*i)->isIsland())
			Broodwar->drawCircle(CoordinateType::Map,c.x(),c.y(),80,Colors::Yellow,false);
	}

	//we will iterate through all the regions and draw the polygon outline of it in green.
	for(std::set<BWTA::Region*>::const_iterator r=BWTA::getRegions().begin();r!=BWTA::getRegions().end();r++)
	{
		BWTA::Polygon p=(*r)->getPolygon();
		for(int j=0;j<(int)p.size();j++)
		{
			Position point1=p[j];
			Position point2=p[(j+1) % p.size()];
			Broodwar->drawLine(CoordinateType::Map,point1.x(),point1.y(),point2.x(),point2.y(),Colors::Green);
		}
	}

	//we will visualize the chokepoints with red lines
	for(std::set<BWTA::Region*>::const_iterator r=BWTA::getRegions().begin();r!=BWTA::getRegions().end();r++)
	{
		for(std::set<BWTA::Chokepoint*>::const_iterator c=(*r)->getChokepoints().begin();c!=(*r)->getChokepoints().end();c++)
		{
			Position point1=(*c)->getSides().first;
			Position point2=(*c)->getSides().second;
			Broodwar->drawLine(CoordinateType::Map,point1.x(),point1.y(),point2.x(),point2.y(),Colors::Red);
		}
	}
}

void ExampleAIModule::showPlayers()
{
	std::set<Player*> players=Broodwar->getPlayers();
	for(std::set<Player*>::iterator i=players.begin();i!=players.end();i++)
	{
		Broodwar->printf("Player [%d]: %s is in force: %s",(*i)->getID(),(*i)->getName().c_str(), (*i)->getForce()->getName().c_str());
	}
}

void ExampleAIModule::showForces()
{
	std::set<Force*> forces=Broodwar->getForces();
	for(std::set<Force*>::iterator i=forces.begin();i!=forces.end();i++)
	{
		std::set<Player*> players=(*i)->getPlayers();
		Broodwar->printf("Force %s has the following players:",(*i)->getName().c_str());
		for(std::set<Player*>::iterator j=players.begin();j!=players.end();j++)
		{
			Broodwar->printf("  - Player [%d]: %s",(*j)->getID(),(*j)->getName().c_str());
		}
	}
}

void ExampleAIModule::onUnitComplete(BWAPI::Unit *unit)
{
	/* if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1) return;
	if (unit->getPlayer() == Broodwar->self()&& unit->getType()==UnitTypes::Terran_SCV){
	Broodwar->printf("scv produced!");
	}

	if (unit->getPlayer() == Broodwar->self()&& unit->getType()==UnitTypes::Terran_Armory){
	buildOrderManager->upgrade(1,UpgradeTypes::Terran_Vehicle_Weapons,20);
	}*/

	//Broodwar->sendText("A %s [%x] has been completed at (%d,%d)",unit->getType().getName().c_str(),unit,unit->getPosition().x(),unit->getPosition().y());
}
